clear all;
close all;
% boundary function
% MUST BE EDITED 
s = @(x) x.^2+x.^3/10-2*x.^4;

%%%%%%%%%%%%%%%%%%%%%%%%%%
% Single Layer Potential %
%%%%%%%%%%%%%%%%%%%%%%%%%%
A = load('data/QBX/SLP/error.mat').Expression1;
nA = load('data/QBX/SLP/Nconvergence.mat').Expression1;
B = load('data/QB2X/SLP/error.mat').Expression1;
nB = load('data/QB2X/SLP/Nconvergence.mat').Expression1;
evalPoints = load('data/QB2X/SLP/dataxy.mat').Expression1;

x=real(evalPoints);
y=imag(evalPoints);

% Interpolate points along the boundary function
xb=linspace(min(x), max(x), 3000);
yb=s(xb);

% Create the interpolating mesh
dx=(max(x)-min(x))/length(A);
dy=(max(y)-min(y))/length(A);
[X,Y]=meshgrid(min(x):dx:max(x),min(y):dy:max(y));

% Create the figure layout
figure('Units','inch','Position',[0 0 15 7]);
movegui(gcf,'west')
fig = tiledlayout(2,5,'TileSpacing','Compact','Padding','Compact');

% Graph the convergence of N
p = nexttile(fig,4,[2 2]);
semilogy(0:length(nA)-1,nA,'linewidth',2,'lineStyle','--');
hold on;
semilogy(0:length(nB)-1,nB,'linewidth',2,'lineStyle','-');
legend(sprintf("QBX Method"),sprintf("QB2X Method"),'FontSize',16,'Interpreter','latex');
xlabel("$N$",'Interpreter','latex');
p.FontSize = 16;

% Plot QBX
kk = [round(length(nA)/3)-1 round(2*length(nA)/3)-1 length(nA)-1];
lowRange=0;
highRange=-Inf;
for n=1:length(kk)
    sub(n) = nexttile(fig);
    m=A(kk(n)+1,:)';
    m(isinf(m)|isnan(m))=-16;
    F=scatteredInterpolant(x,y,m,'natural','nearest');
    Z = F(X,Y);
    Z(Y>=s(X)) = NaN;
    surf(X,Y,Z); view(2);
    hold on
    plot(xb,yb,'k-', 'LineWidth',2);
    grid off;
    axis equal;
    shading flat;
    title(sprintf('$N = %d$',kk(n)), 'Interpreter','latex');
    subtitle(sprintf('$\\|\\log|err|\\|_{\\infty} = %.3f$', log(nA(kk(n)+1))/log(10)),'Interpreter','latex');
    if max(max(Z)) > highRange
        highRange=max(max(Z));
    end
    if min(min(Z)) < lowRange
        lowRange=min(min(Z));
    end
end

% Plot QB2X
kk = [round(length(nB)/3)-1 round(2*length(nB)/3)-1 length(nB)-1];
lowRange=0;
highRange=-Inf;
for n=1:length(kk)
    sub(n+length(kk)) = nexttile(fig);
    m=B(kk(n)+1,:)';
    m(isinf(m)|isnan(m))=-16;
    F=scatteredInterpolant(x,y,m,'natural','nearest');
    Z = F(X,Y);
    Z(Y>=s(X)) = NaN;
    surf(X,Y,Z); view(2);
    hold on
    plot(xb,yb,'k-', 'LineWidth',2);
    grid off;
    axis equal;
    shading flat;
    title(sprintf('$N = %d$',kk(n)), 'Interpreter','latex');
    subtitle(sprintf('$\\|\\log|err|\\|_{\\infty} = %.3f$', log(nB(kk(n)+1))/log(10)),'Interpreter','latex');
    if max(max(Z)) > highRange
        highRange=max(max(Z));
    end
    if min(min(Z)) < lowRange
        lowRange=min(min(Z));
    end
end

% Set plot properties
set(sub, 'Colormap', jet, 'Box', 'on', 'XLim', [min(x),max(x)], 'YLim', [min(y),max(y)], 'CLim', [-16 0], 'FontSize',16)
ylabel(sub(1),'QBX Method');
ylabel(sub(4),'QB2X Method');
rr=colorbar(sub(end));
set( rr, 'YDir', 'reverse' );
rr.Layout.Tile='south';
rr.Title.String = 'Max Log Error';
rr.FontSize = 16; 
exportgraphics(fig,'plots/SLP.pdf','Resolution',300)