(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 12.2' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       158,          7]
NotebookDataLength[    125728,       2602]
NotebookOptionsPosition[    123577,       2563]
NotebookOutlinePosition[    124106,       2582]
CellTagsIndexPosition[    124063,       2579]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
   "Classical", " ", "QBX", " ", "method", " ", "solving", " ", "for", " ", 
    "the", " ", "Helmholtz", " ", "equation", " ", "using", " ", "Hankel", 
    " ", "functions", "\[IndentingNewLine]", "An", " ", "attempt", " ", "at", 
    " ", "reproducing", " ", "similar", " ", "results", " ", "found", " ", 
    "in", " ", "paper", " ", "\"\<Quadrature by expansion...\>\"", 
    "\[IndentingNewLine]", "from", " ", "Klockner", " ", "et", " ", 
    RowBox[{"al", "."}]}], " ", "*)"}], "\[IndentingNewLine]", 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"Unprotect", "[", "Power", "]"}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"Power", "[", 
      RowBox[{
       RowBox[{"0", "|", "0."}], ",", 
       RowBox[{"0", "|", "0."}]}], "]"}], "=", "1"}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Protect", "[", "Power", "]"}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"SetDirectory", "[", 
     RowBox[{"NotebookDirectory", "[", "]"}], "]"}], ";"}], 
   "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{"Prescribe", " ", "conditions", " ", "of", " ", "environment"}], 
    " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"k", "=", "1"}], ";"}], " ", 
   RowBox[{"(*", " ", 
    RowBox[{"wave", " ", "number"}], " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"rho", "[", "s_", "]"}], ":=", "1"}], ";"}], " ", 
   RowBox[{"(*", " ", 
    RowBox[{"Density", " ", "function"}], " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"sc", "[", "s_", "]"}], ":=", 
     RowBox[{
      RowBox[{"s", "^", "2"}], "+", 
      RowBox[{
       RowBox[{"s", "^", "3"}], "/", "10"}], "-", 
      RowBox[{"2", "*", 
       RowBox[{"s", "^", "4"}]}]}]}], ";"}], " ", 
   RowBox[{"(*", " ", 
    RowBox[{"Parameterized", " ", "boundary"}], " ", "*)"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"scp", "[", "s_", "]"}], ":=", 
     RowBox[{
      RowBox[{"sc", "'"}], "[", "s", "]"}]}], ";"}], " ", 
   RowBox[{"(*", " ", 
    RowBox[{"Gradient", " ", "of", " ", "boundary"}], " ", "*)"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"z", "[", "s_", "]"}], ":=", 
     RowBox[{"Sqrt", "[", 
      RowBox[{"1", "+", 
       RowBox[{
        RowBox[{"scp", "[", "s", "]"}], "^", "2"}]}], "]"}]}], ";"}], " ", 
   RowBox[{"(*", " ", 
    RowBox[{"Jacobian", " ", "for", " ", "polar", " ", "transform"}], " ", 
    "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"c", " ", "=", 
     RowBox[{"0", "-", 
      RowBox[{"I", "*", 
       FractionBox["1", "3"]}]}]}], ";"}], " ", 
   RowBox[{"(*", " ", 
    RowBox[{
    "Leaf", " ", "box", " ", "expansion", " ", "center", " ", "for", " ", 
     "demonstration"}], " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"n", "=", " ", "20"}], ";"}], " ", 
   RowBox[{"(*", " ", 
    RowBox[{
    "highest", " ", "number", " ", "of", " ", "expansion", " ", "terms"}], 
    " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"g", "=", " ", "20"}], ";"}], " ", 
   RowBox[{"(*", " ", 
    RowBox[{
    "number", " ", "of", " ", "grid", " ", "points", " ", "along", " ", "x", 
     " ", "and", " ", "y", " ", "axes"}], " ", "*)"}], 
   "\[IndentingNewLine]"}]}]], "Input",
 CellChangeTimes->{{3.824201549034298*^9, 3.8242017306629705`*^9}, 
   3.824201776641529*^9, {3.8242022613614855`*^9, 3.8242022866580477`*^9}, {
   3.8242041709246683`*^9, 3.824204170971409*^9}, {3.8242047032598925`*^9, 
   3.824204711464141*^9}, 3.8242048071184616`*^9, {3.8242059692585754`*^9, 
   3.824205970590394*^9}, {3.82420613069882*^9, 3.8242061353657885`*^9}, {
   3.824206354492281*^9, 3.8242063575644627`*^9}, {3.8242065229371753`*^9, 
   3.824206532063552*^9}, {3.824216395748816*^9, 3.8242164272106795`*^9}, {
   3.824216826990882*^9, 3.824216828866973*^9}, {3.8248106625310774`*^9, 
   3.8248106626601257`*^9}, {3.8248107940125904`*^9, 3.8248107940653563`*^9}, 
   3.8248111836351447`*^9, 3.824815787479703*^9, {3.8248168306747646`*^9, 
   3.82481683078471*^9}, {3.8254080059647636`*^9, 3.8254080061492243`*^9}, {
   3.8260119645833836`*^9, 3.8260119655700293`*^9}, {3.826012169409006*^9, 
   3.8260121810762*^9}, {3.8260126247049007`*^9, 3.826012624801651*^9}, 
   3.8260126634906626`*^9, {3.8260131752408466`*^9, 3.826013189292863*^9}, {
   3.8260136102958045`*^9, 3.826013623485218*^9}, {3.826014012848959*^9, 
   3.82601402435367*^9}, 3.826027053447894*^9, 3.82602727927706*^9, 
   3.826615296514715*^9, 3.826615871082714*^9, {3.826616319064245*^9, 
   3.8266163191172314`*^9}, 3.8266289118377085`*^9, 3.826629262889695*^9, {
   3.8266294841020746`*^9, 3.826629484173549*^9}, {3.859115131535425*^9, 
   3.859115145427741*^9}, {3.859115260951663*^9, 3.8591152622088704`*^9}, {
   3.859895910927072*^9, 3.859895939057004*^9}, 3.859896054134993*^9, 
   3.85989871262799*^9, 3.859899756568331*^9, {3.8629054836069136`*^9, 
   3.862905492965272*^9}, {3.8629059248390064`*^9, 3.8629059259781837`*^9}, 
   3.8629070323324485`*^9, {3.8629087068457103`*^9, 3.862908707059265*^9}, {
   3.862909204798505*^9, 3.862909206503646*^9}, {3.862909765826353*^9, 
   3.862909767981344*^9}, {3.862910654865882*^9, 3.8629106579745955`*^9}, {
   3.8629109193586617`*^9, 3.862910919686694*^9}, {3.862918612462071*^9, 
   3.862918641758218*^9}, 3.862919733771486*^9, 3.862920161891101*^9, 
   3.8629202015304966`*^9, 3.862921154822416*^9, {3.8629220004618025`*^9, 
   3.862922007419752*^9}, {3.8634628746085806`*^9, 3.8634628961666474`*^9}, 
   3.8641107480281377`*^9, {3.864112610767273*^9, 3.8641126395766673`*^9}, {
   3.864113165365574*^9, 3.864113167254612*^9}, 3.864317879485265*^9, {
   3.864477901725404*^9, 3.864477908801912*^9}, {3.8644785551191254`*^9, 
   3.8644785551910267`*^9}, {3.864478603472185*^9, 3.864478604668239*^9}, 
   3.864551995917081*^9, {3.864552037280597*^9, 3.86455211032366*^9}, {
   3.8645522431489735`*^9, 3.8645522457155004`*^9}, 3.8645523049343505`*^9, 
   3.864552662463707*^9, {3.864553273993354*^9, 3.864553274446148*^9}, 
   3.8660954907449923`*^9, 3.8660957839563603`*^9, 3.866095982308855*^9, {
   3.866096180805931*^9, 3.8660962345722947`*^9}, {3.8660968641222615`*^9, 
   3.86609688781585*^9}, 3.866097089007806*^9, {3.8660974909862423`*^9, 
   3.866097509494171*^9}, {3.867920174317848*^9, 3.867920199107699*^9}, {
   3.8679202866043*^9, 3.8679203105027466`*^9}, 3.867921656373554*^9, 
   3.867921941526967*^9, 3.867922088561864*^9, 3.8679223619229727`*^9},
 CellLabel->"In[50]:=",ExpressionUUID->"f82bcb07-fb1f-46be-9b92-f33ee38db7aa"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", 
   RowBox[{"**", 
    RowBox[{
    "*", " ", "Coordinate", " ", "transform", " ", "helper", " ", 
     "functions"}]}], " ", "****)"}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
   "Polar", " ", "coordinate", " ", "pair", " ", "of", " ", "boundary", " ", 
    "point", " ", 
    RowBox[{"w", ".", "r", ".", "t", ".", " ", "c"}]}], " ", "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"rz", "[", "tau_", "]"}], ":=", 
     RowBox[{"Sqrt", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"tau", "-", 
          RowBox[{"Re", "[", "c", "]"}]}], ")"}], "^", "2"}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"sc", "[", "tau", "]"}], "-", 
          RowBox[{"Im", "[", "c", "]"}]}], ")"}], "^", "2"}]}], "]"}]}], 
    ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"argz", "[", "tau_", "]"}], ":=", 
     RowBox[{"Arg", "[", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"tau", "-", 
         RowBox[{"Re", "[", "c", "]"}]}], ")"}], "+", 
       RowBox[{"I", "*", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"sc", "[", "tau", "]"}], " ", "-", " ", 
          RowBox[{"Im", "[", "c", "]"}]}], ")"}]}]}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
    "Polar", " ", "coordinate", " ", "pair", " ", "of", " ", "target", " ", 
     "point", " ", 
     RowBox[{"w", ".", "r", ".", "t", ".", " ", "c"}]}], " ", "*)"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"rw", "[", "weval_", "]"}], ":=", 
     RowBox[{"Sqrt", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Re", "[", "weval", "]"}], "-", 
          RowBox[{"Re", "[", "c", "]"}]}], ")"}], "^", "2"}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Im", "[", "weval", "]"}], "-", 
          RowBox[{"Im", "[", "c", "]"}]}], ")"}], "^", "2"}]}], "]"}]}], 
    ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"argw", "[", "weval_", "]"}], ":=", 
     RowBox[{"Arg", "[", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"Re", "[", "weval", "]"}], "-", 
         RowBox[{"Re", "[", "c", "]"}]}], ")"}], "+", 
       RowBox[{"I", "*", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Im", "[", "weval", "]"}], "-", 
          RowBox[{"Im", "[", "c", "]"}]}], ")"}]}]}], "]"}]}], 
    ";"}]}]}]], "Input",
 CellChangeTimes->{{3.864112227020434*^9, 3.8641122320599575`*^9}, {
  3.867920207461515*^9, 3.867920221657995*^9}},
 CellLabel->"In[62]:=",ExpressionUUID->"69b602e8-6ddf-420a-96b8-feceb41bf3cf"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{"(*", 
   RowBox[{
    RowBox[{
    "**", "**", "**", "**", "**", "**", "**", "**", "**", "**", "**", "**", "**",
      "**", "**", "**", "\[IndentingNewLine]", "Create"}], " ", "mesh", " ", 
    "over", " ", "the", " ", 
    RowBox[{"interval", "\[IndentingNewLine]", "**", "**", "**"}], "*", "*", 
    RowBox[{"**", "**", "**", "**", "**", "**", "**", "**"}]}], "********)"}],
   "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"createMesh", "[", 
      RowBox[{"xlim1_", ",", "xlim2_", ",", "ylim1_", ",", "ylim2_"}], "]"}], 
     ":=", "\[IndentingNewLine]", 
     RowBox[{"tmp", "=", 
      RowBox[{"Select", "[", 
       RowBox[{
        RowBox[{"Flatten", "[", 
         RowBox[{
          RowBox[{"ParallelTable", "[", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{"g", "-", "1", "-", "i"}], ")"}], "/", 
                RowBox[{"(", 
                 RowBox[{"g", "-", "1"}], ")"}]}], ")"}], "*", "xlim1"}], "+", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{"i", "/", 
                RowBox[{"(", 
                 RowBox[{"g", "-", "1"}], ")"}]}], ")"}], "*", "xlim2"}], "+", 
             RowBox[{"I", "*", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{"g", "-", "1", "-", "j"}], ")"}], "/", 
                RowBox[{"(", 
                 RowBox[{"g", "-", "1"}], ")"}]}], ")"}], "*", "ylim1"}], "+", 
             RowBox[{"I", "*", 
              RowBox[{"(", 
               RowBox[{"j", "/", 
                RowBox[{"(", 
                 RowBox[{"g", "-", "1"}], ")"}]}], ")"}], "*", "ylim2"}]}], 
            ",", 
            RowBox[{"{", 
             RowBox[{"i", ",", "0", ",", 
              RowBox[{"g", "-", "1"}]}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"j", ",", "0", ",", 
              RowBox[{"g", "-", "1"}]}], "}"}]}], "]"}], ",", "1"}], "]"}], 
        ",", 
        RowBox[{
         RowBox[{
          RowBox[{"Im", "[", "#", "]"}], "\[NotEqual]", 
          RowBox[{"sc", "[", 
           RowBox[{"Re", "[", "#", "]"}], "]"}]}], "&"}]}], "]"}]}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"wevalPoints", "=", 
     RowBox[{"createMesh", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "1"}], "/", "3"}], ",", 
       RowBox[{"1", "/", "3"}], ",", 
       RowBox[{
        RowBox[{"-", "1"}], "/", "2"}], ",", 
       RowBox[{"1", "/", "6"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{"Length", "[", "wevalPoints", "]"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"p1", "=", 
     RowBox[{"Plot", "[", 
      RowBox[{
       RowBox[{"sc", "[", "s", "]"}], ",", " ", 
       RowBox[{"{", 
        RowBox[{"s", ",", 
         RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", " ", 
       RowBox[{"PlotRange", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"1.1", "*", 
             RowBox[{"Min", "[", 
              RowBox[{"Re", "[", "wevalPoints", "]"}], "]"}]}], ",", 
            RowBox[{"1.1", "*", 
             RowBox[{"Max", "[", 
              RowBox[{"Re", "[", "wevalPoints", "]"}], "]"}]}]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"1.1", "*", 
             RowBox[{"Min", "[", 
              RowBox[{"Im", "[", "wevalPoints", "]"}], "]"}]}], ",", 
            RowBox[{"1.1", "*", 
             RowBox[{"Max", "[", 
              RowBox[{"Im", "[", "wevalPoints", "]"}], "]"}]}]}], "}"}]}], 
         "}"}]}], ",", "  ", 
       RowBox[{"Axes", "\[Rule]", "True"}]}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"p2", "=", 
     RowBox[{"Graphics", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"PointSize", "[", "Large", "]"}], ",", " ", 
        RowBox[{"Point", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Re", "[", "c", "]"}], ",", " ", 
           RowBox[{"Im", "[", "c", "]"}]}], "}"}], "]"}]}], "}"}], "]"}]}], 
    ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"p3", "=", 
     RowBox[{"ListPlot", "[", 
      RowBox[{"Thread", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"Re", "[", "wevalPoints", "]"}], ",", 
         RowBox[{"Im", "[", "wevalPoints", "]"}]}], "}"}], "]"}], "]"}]}], 
    ";"}], "\[IndentingNewLine]", 
   RowBox[{"Show", "[", 
    RowBox[{"p1", ",", "p2", ",", "p3"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.86290552503234*^9, 3.8629055341715975`*^9}, {
   3.862907045492211*^9, 3.86290705499831*^9}, {3.8629071285026073`*^9, 
   3.862907139499606*^9}, {3.8629098157040553`*^9, 3.8629098534925613`*^9}, 
   3.8629186501361322`*^9, {3.8629194231346087`*^9, 3.862919424864961*^9}, {
   3.862920172134658*^9, 3.8629201732778378`*^9}, {3.862920211223621*^9, 
   3.862920221225622*^9}, {3.8629211599615893`*^9, 3.8629211639970884`*^9}, {
   3.862921560685911*^9, 3.8629215718557925`*^9}, {3.8629220209578886`*^9, 
   3.8629220222204413`*^9}, {3.86346290572367*^9, 3.863462908980229*^9}, {
   3.8641107317540913`*^9, 3.8641107320387783`*^9}, {3.8641107750673776`*^9, 
   3.86411077523769*^9}, 3.864112223429037*^9, {3.86431788709599*^9, 
   3.864317902732391*^9}, {3.8644786132297144`*^9, 3.8644786167860384`*^9}, {
   3.864552127698085*^9, 3.8645521342804985`*^9}, {3.8660965472468348`*^9, 
   3.866096555119257*^9}, 3.866096602551182*^9, {3.8660969121915593`*^9, 
   3.86609695016201*^9}, {3.866097514839942*^9, 3.8660975216833096`*^9}, {
   3.8679202345126038`*^9, 3.867920235404002*^9}},
 CellLabel->"In[66]:=",ExpressionUUID->"c120534d-ad3d-4b28-a52a-23a177e3226c"],

Cell[BoxData["400"], "Output",
 CellChangeTimes->{
  3.862905540804943*^9, 3.8629065911462297`*^9, {3.862907113243218*^9, 
   3.8629071403986745`*^9}, 3.8629071782874813`*^9, 3.8629090466608944`*^9, 
   3.8629097800125494`*^9, 3.862909855875724*^9, 3.86291065982838*^9, 
   3.8629109216373515`*^9, 3.862918651682719*^9, 3.862919429545143*^9, 
   3.862919736218258*^9, {3.8629201872455654`*^9, 3.8629202281983323`*^9}, 
   3.862921164861785*^9, 3.862921574940336*^9, 3.862922028002224*^9, 
   3.863462917542016*^9, 3.864110776741166*^9, 3.86411087515793*^9, 
   3.864112283994911*^9, {3.8641126243142643`*^9, 3.8641126423684287`*^9}, 
   3.8641131756218963`*^9, 3.8643179036150265`*^9, 3.8644779923621655`*^9, 
   3.8644781066406484`*^9, 3.86447820511071*^9, 3.86447855881993*^9, 
   3.8644786225375896`*^9, 3.8645521349863997`*^9, 3.8645523949008055`*^9, 
   3.8645525645555964`*^9, 3.864552670722847*^9, 3.864553282683213*^9, 
   3.8660955712229204`*^9, 3.8660957880746145`*^9, 3.8660959864971266`*^9, 
   3.8660962753145733`*^9, {3.866096550855706*^9, 3.866096555844788*^9}, {
   3.8660966036130166`*^9, 3.8660966084338627`*^9}, {3.866096914058324*^9, 
   3.8660969507684507`*^9}, 3.8660970923403473`*^9, 3.866097525672941*^9, 
   3.8679203211905203`*^9, 3.867920954244337*^9, 3.8679211980697703`*^9, 
   3.8679212981952047`*^9, 3.8679216600812716`*^9, 3.8679219450101433`*^9, 
   3.8679223107752714`*^9, 3.8679257065260963`*^9},
 CellLabel->"Out[68]=",ExpressionUUID->"580a52f7-9e7e-4cc8-830d-750d6d7a293e"],

Cell[BoxData[
 GraphicsBox[{{{{}, {}, 
     TagBox[
      {RGBColor[0.368417, 0.506779, 0.709798], AbsoluteThickness[1.6], 
       Opacity[1.], LineBox[CompressedData["
1:eJw1mnk4Vd/3xwkZPkIiRCUhpDJlCqtkSgghU0iaJCFE5iHJTAiZ5ylkHi77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        "]]},
      Annotation[#, "Charting`Private`Tag$6676#1"]& ]}, {}}, 
   {PointSize[Large], 
    PointBox[NCache[{0, Rational[-1, 3]}, {0, -0.3333333333333333}]]}, {{}, 
    {RGBColor[0.368417, 0.506779, 0.709798], PointSize[0.007333333333333334], 
     AbsoluteThickness[1.6], PointBox[CompressedData["
1:eJxd2C+o3nUYxuGjrBpmWBXBYLAYNBi8w8IMBhEMGhYEsQgurBgUNFgMKiwJ
BsuCZUGRBWWKDg0KwylDnGAxWC2rbu+76w73KYfP4bx/uODL9/k9j7765ouv
PXhycvLRAycn936/dPj5/drJ4efva/rh2+cuPPLdX+3Hrn7497Pv/dl++tKt
F86f/aP93N3/fvvUrfYrz7/+5KfXf2u/8fiVz75+/9f2O6funL597kb73qvv
nP65ffath/556tKP7XvvdvHM9+0zN1+++wnftM8fXvBl+/DxFy+3D1/vzrvt
w9f575Pow9e5cKX9zAf/PnH5i6vtw9e59W37+Pt6jt/rZv00P81P89P8ND/N
T/PT/DQ/zU/z0/w0P81P89P8ND/NT/M7/v1G/TQ/zU/z0/w0P81P89P8ND/N
T/PT/DQ/zU/z0/w0P81P8zt+j1/qp/lpfpqf5qf5aX6an+an+Wl+mp/mp/lp
fpqf5qf5aX6a3/H/fqqf5qf5aX6an+an+Wl+mp/mp/lpfpqf5qf5aX6an+an
+Wl+x8/9oX6an+an+Wl+mp/mp/lpfpqf5qf5aX6an+an+Wl+mp/mp/kdX3et
fpqf5qf5aX6an+an+Wl+mp/mp/lpfpqf5qf5aX6an+an+R0/56v6aX6an+an
+Wl+mp/mp/lpfpqf5qf5aX6an+an+Wl+mp/md3yfz+un+Wl+mp/mp/lpfpqf
5qf5aX6an+an+Wl+mp/mp/lpfprf8X0/rp/mp/lpfpqf5qf5aX6an+an+Wl+
mp/mp/lpfpqf5qf56fHL+GX8Mn4Zv4xfxi/jl/HL+GX8Mn4Zv4xfxi/jl/HL
+GX8Mn6Z85s5v5nzmzm/mfObOb+Z85s5v5nzWz895zdzfjPnN3N+M+e3fnrO
b+b8Zs5v5vxm7o/M/ZG5PzL3R+b+yNwfmfsjc39k7o/66bk/6qfn/sjcH/XT
c3/UT8/9kbk/MvdHZn7JzC+Z+SUzv2Tml8z8kplfMvNLZn6pn575pX565pfM
/FI/PfNL/fTML5n5JTO/1O/+PFg/PfNzZn7OzM+Z+TkzP2fm58z8XD8983P9
9MzPmfm5fnrm5/rpmZ8z83P99Dy/1U/P81vm+S3z/JZ5fss8v2We3zLPb/XT
8/xWPz3Pb/XT8/xWPz3Pb/XT8/xWPz37g/rp2R9k9geZ/UFmf5DZH2T2B5n9
Qf307A/qp2d/UD89+4P66dkf1E/P/qB+evZX9dOzv8rsrzL7q8z+KrO/yuyv
Mvur+unZX9VPz/6qfnr2V/XTs7+qn579Vf307E/rp2d/mtmfZvanmf1pZn+a
2Z9m9qf107M/rZ+e/Wn9ND89+9P66dmf1k/z0/zu76Prp2d/n9nfZ/b3mf19
Zn+f2d9n9vf107O/r5+e/X39ND89+/v66dnf10/z0/z+B1xqRis=
      "]]}, {{}, {}}}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->{True, True},
  AxesLabel->{None, None},
  AxesOrigin->{0, 0},
  DisplayFunction->Identity,
  Frame->{{False, False}, {False, False}},
  FrameLabel->{{None, None}, {None, None}},
  FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
  GridLines->{None, None},
  GridLinesStyle->Directive[
    GrayLevel[0.5, 0.4]],
  ImagePadding->All,
  Method->{
   "DefaultBoundaryStyle" -> Automatic, 
    "DefaultGraphicsInteraction" -> {
     "Version" -> 1.2, "TrackMousePosition" -> {True, False}, 
      "Effects" -> {
       "Highlight" -> {"ratio" -> 2}, "HighlightPoint" -> {"ratio" -> 2}, 
        "Droplines" -> {
         "freeformCursorMode" -> True, 
          "placement" -> {"x" -> "All", "y" -> "None"}}}}, "DefaultMeshStyle" -> 
    AbsolutePointSize[6], "ScalingFunctions" -> None, 
    "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
        (Identity[#]& )[
         Part[#, 1]], 
        (Identity[#]& )[
         Part[#, 2]]}& ), "CopiedValueFunction" -> ({
        (Identity[#]& )[
         Part[#, 1]], 
        (Identity[#]& )[
         Part[#, 2]]}& )}},
  PlotRange->{{-0.3666666666666667, 0.3666666666666667}, {-0.55, 
   0.18333333333333335`}},
  PlotRangeClipping->True,
  PlotRangePadding->{{0, 0}, {0, 0}},
  Ticks->{Automatic, Automatic}]], "Output",
 CellChangeTimes->{
  3.862905540804943*^9, 3.8629065911462297`*^9, {3.862907113243218*^9, 
   3.8629071403986745`*^9}, 3.8629071782874813`*^9, 3.8629090466608944`*^9, 
   3.8629097800125494`*^9, 3.862909855875724*^9, 3.86291065982838*^9, 
   3.8629109216373515`*^9, 3.862918651682719*^9, 3.862919429545143*^9, 
   3.862919736218258*^9, {3.8629201872455654`*^9, 3.8629202281983323`*^9}, 
   3.862921164861785*^9, 3.862921574940336*^9, 3.862922028002224*^9, 
   3.863462917542016*^9, 3.864110776741166*^9, 3.86411087515793*^9, 
   3.864112283994911*^9, {3.8641126243142643`*^9, 3.8641126423684287`*^9}, 
   3.8641131756218963`*^9, 3.8643179036150265`*^9, 3.8644779923621655`*^9, 
   3.8644781066406484`*^9, 3.86447820511071*^9, 3.86447855881993*^9, 
   3.8644786225375896`*^9, 3.8645521349863997`*^9, 3.8645523949008055`*^9, 
   3.8645525645555964`*^9, 3.864552670722847*^9, 3.864553282683213*^9, 
   3.8660955712229204`*^9, 3.8660957880746145`*^9, 3.8660959864971266`*^9, 
   3.8660962753145733`*^9, {3.866096550855706*^9, 3.866096555844788*^9}, {
   3.8660966036130166`*^9, 3.8660966084338627`*^9}, {3.866096914058324*^9, 
   3.8660969507684507`*^9}, 3.8660970923403473`*^9, 3.866097525672941*^9, 
   3.8679203211905203`*^9, 3.867920954244337*^9, 3.8679211980697703`*^9, 
   3.8679212981952047`*^9, 3.8679216600812716`*^9, 3.8679219450101433`*^9, 
   3.8679223107752714`*^9, 3.867925706573992*^9},
 CellLabel->"Out[72]=",ExpressionUUID->"674e5675-b69e-40e3-873d-fad6a4b929c0"]
}, Open  ]],

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{"(*", 
   RowBox[{
    RowBox[{"**", "**", "**", "**", "**", "**", "**", "**", "**"}], "*", " ", 
    "\[IndentingNewLine]", "Double", " ", "Layer", " ", 
    RowBox[{"Potential", "\[IndentingNewLine]", "**", "**", "**"}], "*", "*", 
    RowBox[{"**", "**"}]}], "*******)"}]}]], "Input",
 CellChangeTimes->{{3.824208017039632*^9, 3.824208018665269*^9}, 
   3.8260144061962023`*^9},
 CellLabel->"In[73]:=",ExpressionUUID->"1eeb517c-5973-4b73-bbc8-23a5703c85c6"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"ndir", "[", "s_", "]"}], ":=", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", 
        RowBox[{"scp", "[", "s", "]"}]}], ",", "1"}], "}"}], "/", 
     RowBox[{"Sqrt", "[", 
      RowBox[{"1", "+", 
       RowBox[{
        RowBox[{"scp", "[", "s", "]"}], "^", "2"}]}], "]"}]}]}], ";"}], " ", 
  RowBox[{"(*", " ", 
   RowBox[{
   "Normal", " ", "for", " ", "directional", " ", "derivative", " ", "at", 
    " ", "s"}], " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"hp", "[", 
     RowBox[{"weval_", ",", "s_"}], "]"}], ":=", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"Re", "[", "weval", "]"}], "-", "s"}], ",", 
       RowBox[{
        RowBox[{"Im", "[", "weval", "]"}], "-", 
        RowBox[{"sc", "[", "s", "]"}]}]}], "}"}], "*", 
     RowBox[{"(", 
      RowBox[{"-", "k"}], ")"}], "*", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"HankelH1", "[", 
         RowBox[{
          RowBox[{"-", "1"}], ",", 
          RowBox[{"k", "*", 
           RowBox[{"Sqrt", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                RowBox[{"Re", "[", "weval", "]"}], "-", "s"}], ")"}], "^", 
              "2"}], " ", "+", " ", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                RowBox[{"Im", "[", "weval", "]"}], "-", 
                RowBox[{"sc", "[", "s", "]"}]}], ")"}], "^", "2"}]}], 
            "]"}]}]}], "]"}], "-", 
        RowBox[{"HankelH1", "[", 
         RowBox[{"1", ",", 
          RowBox[{"k", "*", 
           RowBox[{"Sqrt", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                RowBox[{"Re", "[", "weval", "]"}], "-", "s"}], ")"}], "^", 
              "2"}], " ", "+", " ", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                RowBox[{"Im", "[", "weval", "]"}], "-", 
                RowBox[{"sc", "[", "s", "]"}]}], ")"}], "^", "2"}]}], 
            "]"}]}]}], "]"}]}], ")"}], "/", 
      RowBox[{"(", 
       RowBox[{"2", "*", 
        RowBox[{"Sqrt", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"Re", "[", "weval", "]"}], "-", "s"}], ")"}], "^", "2"}],
           " ", "+", " ", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"Im", "[", "weval", "]"}], "-", 
             RowBox[{"sc", "[", "s", "]"}]}], ")"}], "^", "2"}]}], "]"}]}], 
       ")"}]}]}]}], ";"}], "\[IndentingNewLine]", "        ", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{
    "derivative", " ", "of", " ", "Hankel", " ", "function", " ", "of", " ", 
     "the", " ", "first", " ", "kind", " ", "for", " ", "argument", " ", 
     "k"}], "|", 
    RowBox[{"w", "-", "z"}], "|"}], " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"EDLP", "[", "weval_", "]"}], ":=", 
    RowBox[{
     FractionBox["I", "4"], "*", 
     RowBox[{"NIntegrate", "[", 
      RowBox[{
       RowBox[{
        RowBox[{
         RowBox[{"hp", "[", 
          RowBox[{"weval", ",", "s"}], "]"}], ".", 
         RowBox[{"ndir", "[", "s", "]"}]}], "*", 
        RowBox[{"rho", "[", "s", "]"}], "*", 
        RowBox[{"z", "[", "s", "]"}]}], ",", 
       RowBox[{"{", 
        RowBox[{"s", ",", 
         RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
       RowBox[{"PrecisionGoal", "\[Rule]", "24"}], ",", 
       RowBox[{"WorkingPrecision", "\[Rule]", "50"}], ",", 
       RowBox[{"MaxRecursion", "\[Rule]", "100"}], ",", 
       RowBox[{"AccuracyGoal", "\[Rule]", "24"}]}], "]"}]}]}], ";"}], 
  "\[IndentingNewLine]", "        ", 
  RowBox[{"(*", " ", 
   RowBox[{
   "Definition", " ", "of", " ", "the", " ", "exact", " ", "Double", " ", 
    "Layer", " ", "Potential", " ", "solution", " ", "using", " ", 
    "numerical", " ", "integration"}], " ", "*)"}]}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"dlpValues", "=", 
   RowBox[{"ParallelTable", "[", 
    RowBox[{
     RowBox[{"EDLP", "[", 
      RowBox[{"wevalPoints", "[", 
       RowBox[{"[", "idx", "]"}], "]"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"idx", ",", "1", ",", 
       RowBox[{"Length", "[", "wevalPoints", "]"}]}], "}"}]}], "]"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.8242083059924192`*^9, 3.8242086337074804`*^9}, {
   3.8242087068814116`*^9, 3.8242087150445795`*^9}, {3.824208757663056*^9, 
   3.824208785721319*^9}, {3.8242089298137712`*^9, 3.824208950289696*^9}, {
   3.8242090713487625`*^9, 3.824209073913897*^9}, {3.824216352940856*^9, 
   3.8242163592670417`*^9}, 3.8242165644365253`*^9, {3.82421663133994*^9, 
   3.824216633216838*^9}, 3.8242167104109287`*^9, {3.8248083323003936`*^9, 
   3.8248083385655947`*^9}, {3.826014424238914*^9, 3.8260144410185704`*^9}, {
   3.864111583100664*^9, 3.8641116391817713`*^9}, {3.8641123936606455`*^9, 
   3.864112393829361*^9}, {3.8641125974748755`*^9, 3.8641126024859395`*^9}, {
   3.8679209217259097`*^9, 3.867920922929226*^9}, {3.867921051858757*^9, 
   3.8679210546786685`*^9}, {3.867921531490903*^9, 3.86792154079996*^9}, 
   3.86792158319382*^9, {3.867921917988125*^9, 3.8679219185887275`*^9}},
 CellLabel->"In[74]:=",ExpressionUUID->"95b16ad9-4377-47ec-8371-eab31b1da77b"],

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{"(*", 
   RowBox[{"**", 
    RowBox[{
    "*", " ", "Calculate", " ", "expansion", " ", "coefficients", " ", 
     "about", " ", "c", " ", "using", " ", 
     RowBox[{"Graff", "'"}], "s", " ", "Addition", " ", "Theorem"}]}], " ", 
   "****)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"hp2x", "[", 
     RowBox[{"s_", ",", "l_"}], "]"}], ":=", " ", 
    RowBox[{
     FractionBox["k", "2"], "*", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"HankelH1", "[", 
         RowBox[{
          RowBox[{"l", "-", "1"}], ",", 
          RowBox[{"k", "*", 
           RowBox[{"rz", "[", "s", "]"}]}]}], "]"}], "*", 
        RowBox[{"E", "^", 
         RowBox[{"(", 
          RowBox[{"I", "*", 
           RowBox[{"(", 
            RowBox[{"l", "-", "1"}], ")"}], "*", 
           RowBox[{"argz", "[", "s", "]"}]}], ")"}]}]}], "-", 
       RowBox[{
        RowBox[{"HankelH1", "[", 
         RowBox[{
          RowBox[{"l", "+", "1"}], ",", 
          RowBox[{"k", "*", 
           RowBox[{"rz", "[", "s", "]"}]}]}], "]"}], "*", 
        RowBox[{"E", "^", 
         RowBox[{"(", 
          RowBox[{"I", "*", 
           RowBox[{"(", 
            RowBox[{"l", "+", "1"}], ")"}], "*", 
           RowBox[{"argz", "[", "s", "]"}]}], ")"}]}]}]}], ")"}]}]}], " ", 
   RowBox[{"(*", " ", 
    RowBox[{"x", " ", "Integrand"}], " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"hp2y", "[", 
      RowBox[{"s_", ",", "l_"}], "]"}], ":=", 
     RowBox[{
      RowBox[{"-", 
       FractionBox["k", 
        RowBox[{"2", "*", "I"}]]}], "*", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"HankelH1", "[", 
          RowBox[{
           RowBox[{"l", "-", "1"}], ",", 
           RowBox[{"k", "*", 
            RowBox[{"rz", "[", "s", "]"}]}]}], "]"}], "*", 
         RowBox[{"E", "^", 
          RowBox[{"(", 
           RowBox[{"I", "*", 
            RowBox[{"(", 
             RowBox[{"l", "-", "1"}], ")"}], "*", 
            RowBox[{"argz", "[", "s", "]"}]}], ")"}]}]}], "+", 
        RowBox[{
         RowBox[{"HankelH1", "[", 
          RowBox[{
           RowBox[{"l", "+", "1"}], ",", 
           RowBox[{"k", "*", 
            RowBox[{"rz", "[", "s", "]"}]}]}], "]"}], "*", 
         RowBox[{"E", "^", 
          RowBox[{"(", 
           RowBox[{"I", "*", 
            RowBox[{"(", 
             RowBox[{"l", "+", "1"}], ")"}], "*", 
            RowBox[{"argz", "[", "s", "]"}]}], ")"}]}]}]}], ")"}]}]}], ";"}], 
   " ", 
   RowBox[{"(*", " ", 
    RowBox[{"y", " ", "Integrand"}], " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"hp2", "[", 
      RowBox[{"s_", ",", "l_"}], "]"}], ":=", 
     RowBox[{
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"hp2x", "[", 
          RowBox[{"s", ",", "l"}], "]"}], ",", 
         RowBox[{"hp2y", "[", 
          RowBox[{"s", ",", "l"}], "]"}]}], "}"}], ".", 
       RowBox[{"ndir", "[", "s", "]"}]}], "*", 
      RowBox[{"rho", "[", "s", "]"}], "*", 
      RowBox[{"z", "[", "s", "]"}]}]}], ";"}], " ", 
   RowBox[{"(*", " ", "Integrand", " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"dlpcoef", "[", "l_", "]"}], ":=", 
    RowBox[{
     FractionBox["I", "4"], "*", 
     RowBox[{"NIntegrate", "[", 
      RowBox[{
       RowBox[{"hp2", "[", 
        RowBox[{"s", ",", "l"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"s", ",", 
         RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
       RowBox[{"PrecisionGoal", "\[Rule]", "24"}], ",", 
       RowBox[{"WorkingPrecision", "\[Rule]", "50"}], ",", 
       RowBox[{"MaxRecursion", "\[Rule]", "100"}], ",", 
       RowBox[{"AccuracyGoal", "\[Rule]", "24"}]}], "]"}]}]}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"dlpCoefs", " ", "=", " ", 
     RowBox[{"ParallelTable", "[", 
      RowBox[{
       RowBox[{"dlpcoef", "[", "l", "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"l", ",", 
         RowBox[{"-", "n"}], ",", "n"}], "}"}]}], "]"}]}], ";"}]}]}]], "Input",
 CellChangeTimes->{{3.8242093733179417`*^9, 3.8242095156464376`*^9}, 
   3.824209653960439*^9, {3.8242107298636274`*^9, 3.8242107558032827`*^9}, {
   3.824210791065707*^9, 3.824210872136051*^9}, {3.8242109269114904`*^9, 
   3.8242109786958838`*^9}, 3.8242114049525647`*^9, {3.8242115385360374`*^9, 
   3.8242115522187614`*^9}, {3.8242121301696987`*^9, 3.824212137603359*^9}, {
   3.8242121845478334`*^9, 3.8242121862667427`*^9}, {3.824212271410513*^9, 
   3.8242123430930166`*^9}, {3.8242156223008404`*^9, 
   3.8242156786411686`*^9}, {3.824215710341919*^9, 3.8242157375339417`*^9}, 
   3.8242158237735124`*^9, {3.824215886688427*^9, 3.824215914785874*^9}, {
   3.8242163729466066`*^9, 3.8242163860015955`*^9}, {3.824216514684069*^9, 
   3.8242165213884063`*^9}, {3.826014453675293*^9, 3.826014478827733*^9}, {
   3.8266140423241425`*^9, 3.8266140433640547`*^9}, 3.8266143714870024`*^9, {
   3.862905984920796*^9, 3.8629059967899113`*^9}, {3.8641127476433153`*^9, 
   3.864112747878539*^9}, {3.867920941464795*^9, 3.8679209422601357`*^9}, {
   3.867920989322877*^9, 3.867920989653131*^9}, {3.8679210900647335`*^9, 
   3.8679210923487883`*^9}, {3.8679216124242554`*^9, 3.867921628224266*^9}, {
   3.8679217394733686`*^9, 3.8679217555642157`*^9}, {3.867921803916709*^9, 
   3.8679218170835657`*^9}, {3.867921880809226*^9, 3.8679219325448008`*^9}, {
   3.8679220408891554`*^9, 3.8679220415510015`*^9}, {3.8679221953647256`*^9, 
   3.8679221984369154`*^9}, {3.8679222726864824`*^9, 
   3.8679222759402018`*^9}, {3.8679223069086437`*^9, 3.8679223080087423`*^9}, 
   3.867922394545521*^9},
 CellLabel->"In[78]:=",ExpressionUUID->"397765d1-d60b-4c96-b2d7-b1e6cd4d7a1d"],

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{"(*", 
   RowBox[{"**", 
    RowBox[{
    "*", " ", "Calculate", " ", "the", " ", "approximate", " ", "numerical", 
     " ", "double", " ", "layer", " ", "solution", " ", "using", " ", 
     "expansion", " ", "coefficients"}]}], " ", "****)"}], 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"NDLPpart", "[", 
      RowBox[{"idx_", ",", "nn_"}], "]"}], ":=", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"nn", "\[Equal]", "0"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"dlpCoefs", "[", 
         RowBox[{"[", 
          RowBox[{"n", "+", "1"}], "]"}], "]"}], "*", 
        RowBox[{"BesselJ", "[", 
         RowBox[{"0", ",", 
          RowBox[{"k", "*", 
           RowBox[{"rw", "[", 
            RowBox[{"wevalPoints", "[", 
             RowBox[{"[", "idx", "]"}], "]"}], "]"}]}]}], "]"}], "*", 
        RowBox[{"E", "^", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "I"}], "*", "0", "*", 
           RowBox[{"argw", "[", 
            RowBox[{"wevalPoints", "[", 
             RowBox[{"[", "idx", "]"}], "]"}], "]"}]}], ")"}]}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Sum", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"dlpCoefs", "[", 
           RowBox[{"[", 
            RowBox[{"l", "+", "n", "+", "1"}], "]"}], "]"}], "*", 
          RowBox[{"BesselJ", "[", 
           RowBox[{"l", ",", 
            RowBox[{"k", "*", 
             RowBox[{"rw", "[", 
              RowBox[{"wevalPoints", "[", 
               RowBox[{"[", "idx", "]"}], "]"}], "]"}]}]}], "]"}], "*", 
          RowBox[{"E", "^", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", "I"}], "*", "l", "*", 
             RowBox[{"argw", "[", 
              RowBox[{"wevalPoints", "[", 
               RowBox[{"[", "idx", "]"}], "]"}], "]"}]}], ")"}]}]}], ",", 
         RowBox[{"{", 
          RowBox[{"l", ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", "nn"}], ",", "nn"}], "}"}]}], "}"}]}], "]"}]}], 
      "]"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"ndlpValues", "=", 
     RowBox[{"ParallelTable", "[", 
      RowBox[{
       RowBox[{"NDLPpart", "[", 
        RowBox[{"idx", ",", "nn"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"idx", ",", "1", ",", 
         RowBox[{"Length", "[", "wevalPoints", "]"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"nn", ",", "0", ",", "n"}], "}"}]}], "]"}]}], 
    ";"}]}]}]], "Input",
 CellChangeTimes->{{3.824210083247199*^9, 3.8242101046887274`*^9}, 
   3.824210135995981*^9, {3.826014493935771*^9, 3.826014497086623*^9}, {
   3.8260145459782777`*^9, 3.826014549621372*^9}, {3.826616381390806*^9, 
   3.826616382107689*^9}, 3.8266289628303556`*^9, {3.8629059995534863`*^9, 
   3.8629060048777037`*^9}, {3.8641117339846454`*^9, 
   3.8641120258817983`*^9}, {3.867920969630329*^9, 3.8679210454508934`*^9}, {
   3.8679210873387775`*^9, 3.8679210891840286`*^9}, {3.8679214541392093`*^9, 
   3.8679214550657806`*^9}, {3.867921642625843*^9, 3.8679216459003153`*^9}},
 CellLabel->"In[83]:=",ExpressionUUID->"ac8a513c-d4ca-46b4-905a-405de129923f"],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{"(*", " ", 
   RowBox[{
   "Create", " ", "an", " ", "error", " ", "function", " ", "to", " ", 
    "compare", " ", "the", " ", "exact", " ", "solution", " ", "to", " ", 
    "the", " ", "approximate", " ", "numerical", " ", "solution"}], " ", 
   "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"DLPERR", "[", "Nmax_", "]"}], ":=", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"dlpValues", "-", 
       RowBox[{"Sum", "[", 
        RowBox[{
         RowBox[{"ndlpValues", "[", 
          RowBox[{"[", 
           RowBox[{"All", ",", "nn"}], "]"}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"nn", ",", "1", ",", 
           RowBox[{"Nmax", "+", "1"}]}], "}"}]}], "]"}]}], ")"}], "/", 
     "dlpValues"}]}], ";"}]}]], "Input",
 CellChangeTimes->{
  3.8641115516699758`*^9, {3.8641120723638477`*^9, 3.8641120775115395`*^9}, {
   3.8641129138586826`*^9, 3.864112913988682*^9}, {3.86609754017093*^9, 
   3.8660975446146994`*^9}, {3.8679210751876535`*^9, 
   3.8679210765967894`*^9}, {3.8679211078608294`*^9, 3.8679211120856133`*^9}},
 CellLabel->"In[85]:=",ExpressionUUID->"376a6ed5-f0d3-4f5b-9257-ec0b082b41d5"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{"(*", 
   RowBox[{"**", 
    RowBox[{
    "*", " ", "Create", " ", "a", " ", "graphical", " ", "representation", 
     " ", "of", " ", "the", " ", "error", " ", "choice", " ", "of", " ", 
     "N"}]}], " ", "****)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"err", "=", 
     RowBox[{"DLPERR", "[", "n", "]"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"data", "=", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"Re", "[", 
          RowBox[{"wevalPoints", "[", 
           RowBox[{"[", "idx", "]"}], "]"}], "]"}], ",", 
         RowBox[{"Im", "[", 
          RowBox[{"wevalPoints", "[", 
           RowBox[{"[", "idx", "]"}], "]"}], "]"}], ",", 
         RowBox[{
          RowBox[{"Log", "[", 
           RowBox[{"Abs", "[", 
            RowBox[{"err", "[", 
             RowBox[{"[", "idx", "]"}], "]"}], "]"}], "]"}], "/", 
          RowBox[{"Log", "[", "10", "]"}]}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"idx", ",", "1", ",", 
         RowBox[{"Length", "[", "err", "]"}]}], "}"}]}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"data", "=", 
     RowBox[{"data", "/.", 
      RowBox[{"Indeterminate", "\[Rule]", 
       RowBox[{"-", "16"}]}]}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{"maxError", "=", 
    RowBox[{"Max", "[", 
     RowBox[{
      RowBox[{"Select", "[", 
       RowBox[{"data", ",", 
        RowBox[{
         RowBox[{
          RowBox[{"#", "[", 
           RowBox[{"[", "2", "]"}], "]"}], "<", 
          RowBox[{"sc", "[", 
           RowBox[{"#", "[", 
            RowBox[{"[", "1", "]"}], "]"}], "]"}]}], "&"}]}], "]"}], "[", 
      RowBox[{"[", 
       RowBox[{"All", ",", "3"}], "]"}], "]"}], "]"}]}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"minError", "=", 
     RowBox[{"Min", "[", 
      RowBox[{
       RowBox[{"Select", "[", 
        RowBox[{"data", ",", 
         RowBox[{
          RowBox[{
           RowBox[{"#", "[", 
            RowBox[{"[", "2", "]"}], "]"}], "<", 
           RowBox[{"sc", "[", 
            RowBox[{"#", "[", 
             RowBox[{"[", "1", "]"}], "]"}], "]"}]}], "&"}]}], "]"}], "[", 
       RowBox[{"[", 
        RowBox[{"All", ",", "3"}], "]"}], "]"}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"plot1", "=", 
     RowBox[{"ListDensityPlot", "[", 
      RowBox[{"data", ",", "\[IndentingNewLine]", 
       RowBox[{"RegionFunction", "\[Rule]", 
        RowBox[{"Function", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"x", ",", "y", ",", "z"}], "}"}], ",", 
          RowBox[{"y", "<", 
           RowBox[{"sc", "[", "x", "]"}]}]}], "]"}]}], ",", 
       "\[IndentingNewLine]", " ", 
       RowBox[{"PlotRange", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"1.1", "*", 
             RowBox[{"Min", "[", 
              RowBox[{"Re", "[", "wevalPoints", "]"}], "]"}]}], ",", 
            RowBox[{"1.1", "*", 
             RowBox[{"Max", "[", 
              RowBox[{"Re", "[", "wevalPoints", "]"}], "]"}]}]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"1.1", "*", 
             RowBox[{"Min", "[", 
              RowBox[{"Im", "[", "wevalPoints", "]"}], "]"}]}], ",", 
            RowBox[{"1.1", "*", 
             RowBox[{"Max", "[", 
              RowBox[{"Im", "[", "wevalPoints", "]"}], "]"}]}]}], "}"}], ",", 
          "All"}], "}"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"ColorFunctionScaling", "\[Rule]", "False"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"ColorFunction", "\[Rule]", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"ColorData", "[", "\"\<DarkRainbow\>\"", "]"}], "[", 
           RowBox[{"Rescale", "[", 
            RowBox[{"#", ",", " ", 
             RowBox[{"{", 
              RowBox[{"minError", ",", "maxError"}], "}"}]}], "]"}], "]"}], 
          " ", "&"}], ")"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"PlotLegends", "\[Rule]", 
        RowBox[{"BarLegend", "[", 
         RowBox[{"{", 
          RowBox[{"Automatic", ",", 
           RowBox[{"{", 
            RowBox[{"minError", ",", "maxError"}], "}"}]}], "}"}], "]"}]}]}], 
      "]"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"plot2", "=", 
     RowBox[{"Graphics", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"PointSize", "[", "Large", "]"}], ",", " ", 
        RowBox[{"Point", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Re", "[", "c", "]"}], ",", " ", 
           RowBox[{"Im", "[", "c", "]"}]}], "}"}], "]"}]}], "}"}], "]"}]}], 
    ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"plot3", "=", 
     RowBox[{"Plot", "[", 
      RowBox[{
       RowBox[{"sc", "[", "x", "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", 
         RowBox[{"-", "1"}], ",", "1"}], "}"}]}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{"Show", "[", 
    RowBox[{"plot1", ",", "plot2", ",", "plot3"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.8260175730494905`*^9, 3.8260175875731335`*^9}, {
   3.826017725764667*^9, 3.826017738986579*^9}, {3.8260178523475857`*^9, 
   3.8260179248973093`*^9}, {3.826018581433931*^9, 3.8260185838928394`*^9}, {
   3.8266278422806926`*^9, 3.826627889655772*^9}, {3.842620189256916*^9, 
   3.842620192955665*^9}, {3.8426202463370495`*^9, 3.8426202633717985`*^9}, {
   3.842621023392301*^9, 3.842621027180705*^9}, {3.8591154732117615`*^9, 
   3.859115484974616*^9}, {3.859115710923396*^9, 3.859115726224893*^9}, {
   3.8591157937145205`*^9, 3.85911582271305*^9}, {3.859115888253096*^9, 
   3.859115924136121*^9}, {3.859896095794634*^9, 3.859896096879325*^9}, {
   3.8598961295913277`*^9, 3.859896139559779*^9}, {3.859897978624682*^9, 
   3.859898023988381*^9}, {3.859898652079686*^9, 3.8598986524523697`*^9}, {
   3.8629057254688616`*^9, 3.8629057266622734`*^9}, {3.862908066659073*^9, 
   3.8629080706782465`*^9}, 3.862910264320558*^9, {3.8629103081895604`*^9, 
   3.8629103820593705`*^9}, {3.8629104487036076`*^9, 3.862910449055311*^9}, {
   3.8629108535404935`*^9, 3.8629109034562116`*^9}, {3.862911076883559*^9, 
   3.862911082335763*^9}, {3.862911334898176*^9, 3.8629113364819202`*^9}, {
   3.862918941500255*^9, 3.862918941822801*^9}, {3.8629190836532764`*^9, 
   3.8629191141001663`*^9}, {3.862919337703516*^9, 3.8629193819062657`*^9}, {
   3.862919478060832*^9, 3.8629194788278165`*^9}, {3.862920771338073*^9, 
   3.862920806149024*^9}, {3.8629209575855923`*^9, 3.862920961089507*^9}, {
   3.862921011200637*^9, 3.862921013077097*^9}, {3.8629210458460107`*^9, 
   3.8629210654203663`*^9}, {3.8634630263533435`*^9, 
   3.8634630569106517`*^9}, {3.8641120891408486`*^9, 3.8641120917617383`*^9}, 
   3.8641131930125256`*^9, {3.864113405496626*^9, 3.8641134159438467`*^9}, {
   3.8641134854182467`*^9, 3.864113492501912*^9}, {3.864113546514684*^9, 
   3.8641135466523128`*^9}, {3.8643179967892838`*^9, 3.864317996894004*^9}, {
   3.864478052227618*^9, 3.86447808742808*^9}, {3.864478160442616*^9, 
   3.864478171594324*^9}, {3.8644782181334357`*^9, 3.864478221118528*^9}, {
   3.8644782608965416`*^9, 3.864478272498558*^9}, {3.8644783049774323`*^9, 
   3.8644783592340837`*^9}, {3.864552348121228*^9, 3.8645523574758296`*^9}, {
   3.8645524471703196`*^9, 3.864552455682497*^9}, {3.866097280102521*^9, 
   3.8660972804679413`*^9}, {3.866097316335229*^9, 3.866097316598853*^9}, {
   3.8660975507777195`*^9, 3.8660975577554092`*^9}, {3.8679205062393017`*^9, 
   3.867920528281221*^9}, {3.867920649736471*^9, 3.867920654720306*^9}, {
   3.867920792729556*^9, 3.8679207986355762`*^9}, {3.8679211164245834`*^9, 
   3.867921126639617*^9}, {3.867921295227928*^9, 3.8679212956450405`*^9}, {
   3.8679213938620434`*^9, 3.867921398899049*^9}, {3.867921467093318*^9, 
   3.8679214700637264`*^9}, {3.8679223490596113`*^9, 3.867922358106301*^9}, {
   3.8679243648158593`*^9, 3.8679243868613186`*^9}, {3.867924611968038*^9, 
   3.8679246141082726`*^9}},
 CellLabel->"In[86]:=",ExpressionUUID->"587aaa57-1a31-458b-9a74-f96f79916698"],

Cell[BoxData[
 RowBox[{"-", 
  "4.4034082455049593623512033216036667120315777243330887564986`46.\
22757139050575"}]], "Output",
 CellChangeTimes->{{3.864478318195645*^9, 3.864478360310031*^9}, 
   3.864478687692378*^9, 3.864552383101451*^9, {3.864552451462137*^9, 
   3.8645524578885174`*^9}, 3.8645525410542965`*^9, 3.8645526365965147`*^9, 
   3.8645527273738947`*^9, 3.8645527942486963`*^9, 3.8645531417522535`*^9, 
   3.8645533382915287`*^9, 3.86609569664365*^9, 3.86609584584993*^9, 
   3.866096041082574*^9, 3.866096395871772*^9, 3.8660967332652693`*^9, 
   3.8660970471607695`*^9, 3.866097184300061*^9, {3.8660972773998413`*^9, 
   3.866097280970598*^9}, 3.86609731840666*^9, 3.8660977030523944`*^9, 
   3.8679205308900614`*^9, {3.867920651766673*^9, 3.8679206556158466`*^9}, {
   3.8679207889795876`*^9, 3.867920799149659*^9}, 3.8679212300102973`*^9, 
   3.8679213294903817`*^9, {3.867921394244588*^9, 3.8679213994197626`*^9}, {
   3.8679214625761333`*^9, 3.8679214705182734`*^9}, 3.867921692738534*^9, 
   3.8679218437178288`*^9, 3.8679219770477514`*^9, 3.867922082779477*^9, 
   3.8679221342272167`*^9, 3.8679222230260916`*^9, {3.8679223437519436`*^9, 
   3.867922358480153*^9}, 3.867925737958393*^9},
 CellLabel->"Out[89]=",ExpressionUUID->"e4573d85-d964-488f-85f5-35d7e57333b9"],

Cell[BoxData[
 TemplateBox[{
   GraphicsBox[{
     GraphicsComplexBox[CompressedData["
1:eJx1mH1snlUVwBtFMo0aYXEjQ0UyY8QsyESJMtxBUaaI2M0PHCGNTinqwLK4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      "], {{{
         EdgeForm[], 
         GrayLevel[0.8], 
         GraphicsGroupBox[{
           PolygonBox[CompressedData["
1:eJxNmXWYltUWxed8MNTQMRJDDDDAkI50dwsiIV0KQ4OECBKXlBJJAy8qoaSI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            "]], 
           PolygonBox[CompressedData["
1:eJw11OdvT3EUx/Hv96qSWA0h8ZCH/gXa0BpN/Ai1N61Ro/3Ze8V6JEbsUUL0
Aam9Q4wQo3ZDaGxiBUFSUqLifXI+ffDKub9773ec8z33164wXVCahBBSyMCm
GMJXjMYafi8iLkRfFOAOlmEE+mAHDmIwpuIEWjHhAmJvbMAndMMS5vsSfQ2b
ewDxHu5qrvbc+03cjnPB53yG5chGjp79QZHe+U7cgjKuBxJ34h/GopJ7r4kr
0QW5OK81ba1s5bIOk5TTIxxFCr3QkFxWE3sgH1vxC4WoYI49lgsmYA4OIzBm
OrE7fmqMvTuIWKc92t6GEPfhOIZjvnL5gTHK6aNqaLXrT/yAVeiqmu5GY9Yb
RuyJS7iPfsizOXAo+lrTUI21KNaalXiCtuiAC3iJzpiMKjyIXlvbz3riC/WC
vfNcZ5SjHlmhOWzsSNTgivK3mj1FOTqp5la7G5itGj7EEdV6Jh7jmM5iLi7i
ra0f/EytFu8wRTVZjFMYpRostdphaPQavYneE7l65zLeq1Y2xzdsRqnO4CRe
KRdb8zNxI0qifyONEj8Dq31t8Fyacm+8crK1T2Oc9nAVCc+rg/eI9VYDfs+I
3mPXkJF479i9WmxDOnrP29iy+rMMvpf90XujRLWu0llZzW1sRfTeSyu3A9G/
PcuxCfOd0d5tz3+jf9P2Lds3lMnzXToLO5PWifes9WpHnEUL7s1TDzXnulni
30JKuV/HLNXgluawsfYfkcX1Xp2tnXHLxHvSerFYY20P+ZrjpsbYu7ZmG65v
R/+vmYj/hLOeCA==
            "]]}]}, {}, {}, {}, {}}}, VertexColors -> CompressedData["
1:eJztmfk7VP///xFlCxHRKtkKUZYiuZeKpJKStAkpLUq2ksrSIslWUlmiBSVR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       "]], {
      PointSize[Large], 
      PointBox[
       NCache[{0, 
         Rational[-1, 3]}, {0, -0.3333333333333333}]]}, {{{{}, {}, 
        TagBox[{
          Directive[
           Opacity[1.], 
           RGBColor[0.368417, 0.506779, 0.709798], 
           AbsoluteThickness[1.6]], 
          LineBox[CompressedData["
1:eJw1mnk4Vd/3x0kyfIREiEpCSGXKFJYkJISQKSRNkiRExpBkJoTMZA6ZXZd9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           "]]}, Annotation[#, "Charting`Private`Tag$6926#1"]& ]}}, {}}}, {
    DisplayFunction -> Identity, AspectRatio -> 1, 
     AxesOrigin -> {Automatic, Automatic}, DisplayFunction :> Identity, Frame -> 
     True, FrameLabel -> {None, None}, 
     FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {None, None}, GridLinesStyle -> Directive[
       GrayLevel[0.5, 0.4]], 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, "DefaultColorFunction" -> 
       "M10DefaultDensityGradient", 
       "DefaultGraphicsInteraction" -> {
        "Version" -> 1.2, "TrackMousePosition" -> {True, False}, 
         "Effects" -> {
          "Highlight" -> {"ratio" -> 2}, "HighlightPoint" -> {"ratio" -> 2}, 
           "Droplines" -> {
            "freeformCursorMode" -> True, 
             "placement" -> {"x" -> "All", "y" -> "None"}}}}, 
       "GridLinesInFront" -> True}, 
     PlotRange -> {{-0.3666666666666667, 0.3666666666666667}, {-0.55, 
      0.18333333333333335`}}, PlotRangeClipping -> True, PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.02], 
        Scaled[0.02]}}, Ticks -> {Automatic, Automatic}}], 
   FormBox[
    FormBox[
     TemplateBox[{
       FormBox[
        StyleBox[
         StyleBox[
          PaneBox[
           GraphicsBox[{
             RasterBox[CompressedData["
1:eJztl/k31G/cxokiLcpWRFlThEopkssSRSEiSulbkXwrqVCiCCVLUSFrJCr7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              "], {{
                Rational[-15, 2], 
                Rational[-225, 2]}, {
                Rational[15, 2], 
                Rational[225, 2]}}], {Antialiasing -> False, 
              AbsoluteThickness[0.1], 
              Directive[
               Opacity[0.3], 
               GrayLevel[0]], 
              LineBox[
               NCache[{{
                  Rational[15, 2], 
                  Rational[-225, 2]}, {
                  Rational[-15, 2], 
                  Rational[-225, 2]}, {
                  Rational[-15, 2], 
                  Rational[225, 2]}, {
                  Rational[15, 2], 
                  Rational[225, 2]}, {
                  Rational[15, 2], 
                  Rational[-225, 2]}}, {{7.5, -112.5}, {-7.5, -112.5}, {-7.5, 
                112.5}, {7.5, 112.5}, {7.5, -112.5}}]]}, {
              CapForm[None], {}}, 
             StyleBox[{Antialiasing -> False, 
               StyleBox[
                LineBox[{{7.5, -112.5}, {7.5, 112.5}}], {
                 Directive[
                  AbsoluteThickness[0.2], 
                  Opacity[0.3], 
                  GrayLevel[0]]}, StripOnInput -> False], 
               StyleBox[
                StyleBox[{{
                   StyleBox[
                    LineBox[{{{7.5, -112.5}, 
                    Offset[{4., 0}, {7.5, -112.5}]}, {{7.5, -75.}, 
                    Offset[{4., 0}, {7.5, -75.}]}, {{7.5, -37.5}, 
                    Offset[{4., 0}, {7.5, -37.5}]}, {{7.5, 0.}, 
                    Offset[{4., 0}, {7.5, 0.}]}, {{7.5, 37.5}, 
                    Offset[{4., 0}, {7.5, 37.5}]}, {{7.5, 75.}, 
                    Offset[{4., 0}, {7.5, 75.}]}, {{7.5, 112.5}, 
                    Offset[{4., 0}, {7.5, 112.5}]}}], {
                    Directive[
                    AbsoluteThickness[0.2], 
                    GrayLevel[0.4]]}, StripOnInput -> False], 
                   StyleBox[
                    LineBox[{{{7.5, -105.}, 
                    Offset[{2.5, 0.}, {7.5, -105.}]}, {{7.5, -97.5}, 
                    Offset[{2.5, 0.}, {7.5, -97.5}]}, {{7.5, -90.}, 
                    Offset[{2.5, 0.}, {7.5, -90.}]}, {{7.5, -82.5}, 
                    Offset[{2.5, 0.}, {7.5, -82.5}]}, {{7.5, -67.5}, 
                    Offset[{2.5, 0.}, {7.5, -67.5}]}, {{7.5, -60.}, 
                    Offset[{2.5, 0.}, {7.5, -60.}]}, {{7.5, -52.5}, 
                    Offset[{2.5, 0.}, {7.5, -52.5}]}, {{7.5, -45.}, 
                    Offset[{2.5, 0.}, {7.5, -45.}]}, {{7.5, -30.}, 
                    Offset[{2.5, 0.}, {7.5, -30.}]}, {{7.5, -22.5}, 
                    Offset[{2.5, 0.}, {7.5, -22.5}]}, {{7.5, -15.}, 
                    Offset[{2.5, 0.}, {7.5, -15.}]}, {{7.5, -7.5}, 
                    Offset[{2.5, 0.}, {7.5, -7.5}]}, {{7.5, 7.5}, 
                    Offset[{2.5, 0.}, {7.5, 7.5}]}, {{7.5, 15.}, 
                    Offset[{2.5, 0.}, {7.5, 15.}]}, {{7.5, 22.5}, 
                    Offset[{2.5, 0.}, {7.5, 22.5}]}, {{7.5, 30.}, 
                    Offset[{2.5, 0.}, {7.5, 30.}]}, {{7.5, 45.}, 
                    Offset[{2.5, 0.}, {7.5, 45.}]}, {{7.5, 52.5}, 
                    Offset[{2.5, 0.}, {7.5, 52.5}]}, {{7.5, 60.}, 
                    Offset[{2.5, 0.}, {7.5, 60.}]}, {{7.5, 67.5}, 
                    Offset[{2.5, 0.}, {7.5, 67.5}]}, {{7.5, 82.5}, 
                    Offset[{2.5, 0.}, {7.5, 82.5}]}, {{7.5, 90.}, 
                    Offset[{2.5, 0.}, {7.5, 90.}]}, {{7.5, 97.5}, 
                    Offset[{2.5, 0.}, {7.5, 97.5}]}, {{7.5, 105.}, 
                    Offset[{2.5, 0.}, {7.5, 105.}]}}], {
                    Directive[
                    AbsoluteThickness[0.2], 
                    GrayLevel[0.4], 
                    Opacity[0.3]]}, StripOnInput -> False]}, 
                  StyleBox[
                   StyleBox[{{
                    StyleBox[{
                    InsetBox[
                    FormBox[
                    RowBox[{"-", "35"}], TraditionalForm], 
                    Offset[{7., 0.}, {7.5, -112.5}], 
                    ImageScaled[{0, 0.5}], Automatic, {1, 0}], 
                    InsetBox[
                    FormBox[
                    RowBox[{"-", "30"}], TraditionalForm], 
                    Offset[{7., 0.}, {7.5, -75.}], 
                    ImageScaled[{0, 0.5}], Automatic, {1, 0}], 
                    InsetBox[
                    FormBox[
                    RowBox[{"-", "25"}], TraditionalForm], 
                    Offset[{7., 0.}, {7.5, -37.5}], 
                    ImageScaled[{0, 0.5}], Automatic, {1, 0}], 
                    InsetBox[
                    FormBox[
                    RowBox[{"-", "20"}], TraditionalForm], 
                    Offset[{7., 0.}, {7.5, 0.}], 
                    ImageScaled[{0, 0.5}], Automatic, {1, 0}], 
                    InsetBox[
                    FormBox[
                    RowBox[{"-", "15"}], TraditionalForm], 
                    Offset[{7., 0.}, {7.5, 37.5}], 
                    ImageScaled[{0, 0.5}], Automatic, {1, 0}], 
                    InsetBox[
                    FormBox[
                    RowBox[{"-", "10"}], TraditionalForm], 
                    Offset[{7., 0.}, {7.5, 75.}], 
                    ImageScaled[{0, 0.5}], Automatic, {1, 0}], 
                    InsetBox[
                    FormBox[
                    RowBox[{"-", "5"}], TraditionalForm], 
                    Offset[{7., 0.}, {7.5, 112.5}], 
                    ImageScaled[{0, 0.5}], Automatic, {1, 0}]}, {
                    Directive[
                    AbsoluteThickness[0.2], 
                    GrayLevel[0.4]], {
                    Directive[
                    Opacity[1]], 
                    Directive[
                    Opacity[1]]}}, StripOnInput -> False], 
                    
                    StyleBox[{{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, \
{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}}, {
                    Directive[
                    AbsoluteThickness[0.2], 
                    GrayLevel[0.4], 
                    Opacity[0.3]], {
                    Directive[
                    Opacity[1]], 
                    Directive[
                    Opacity[1]]}}, StripOnInput -> False]}, {}}, {{
                    Directive[
                    Opacity[1]], 
                    Directive[
                    Opacity[1]]}}, StripOnInput -> False], {"GraphicsLabel"}, 
                   StripOnInput -> False]}, {"GraphicsTicks"}, StripOnInput -> 
                 False], {{
                  Directive[
                   AbsoluteThickness[0.2], 
                   GrayLevel[0.4]], 
                  Directive[
                   AbsoluteThickness[0.2], 
                   GrayLevel[0.4], 
                   Opacity[0.3]]}}, StripOnInput -> False]}, {"GraphicsAxes"},
               StripOnInput -> False]}, PlotRangePadding -> Scaled[0.02], 
            PlotRange -> All, Frame -> True, 
            FrameTicks -> {{False, False}, {True, False}}, FrameStyle -> 
            Opacity[0], FrameTicksStyle -> Opacity[0], 
            ImageSize -> {Automatic, 225}, BaseStyle -> {}], Alignment -> 
           Left, AppearanceElements -> None, ImageMargins -> {{5, 5}, {5, 5}},
            ImageSizeAction -> "ResizeToFit"], LineIndent -> 0, StripOnInput -> 
          False], {FontFamily -> "Arial"}, Background -> Automatic, 
         StripOnInput -> False], TraditionalForm]}, "BarLegend", 
      DisplayFunction -> (#& ), 
      InterpretationFunction :> (RowBox[{"BarLegend", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{
               RowBox[{
                 RowBox[{
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"ColorData", "[", "\"DarkRainbow\"", "]"}], "[", 
                    RowBox[{"Rescale", "[", 
                    RowBox[{"#1", ",", 
                    RowBox[{"{", 
                    RowBox[{"minError", ",", "maxError"}], "}"}]}], "]"}], 
                    "]"}], "&"}], ")"}], "[", "#1", "]"}], "&"}], ",", 
               RowBox[{"{", 
                 RowBox[{
                   
                   RowBox[{
                    "-", "35.27038401232596178804903596291407306603`16.\
2808681427034"}], ",", 
                   
                   RowBox[{
                    "-", "4.\
4034082455049593623512033216036667120315777243330887564986`46.22757139050575"}\
]}], "}"}]}], "}"}], ",", 
           RowBox[{"LabelStyle", "\[Rule]", 
             RowBox[{"{", "}"}]}], ",", 
           RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}], ",", 
           RowBox[{"LegendMarkerSize", "\[Rule]", "225"}], ",", 
           RowBox[{"ScalingFunctions", "\[Rule]", 
             RowBox[{"{", 
               RowBox[{"Identity", ",", "Identity"}], "}"}]}], ",", 
           RowBox[{"Charting`TickSide", "\[Rule]", "Right"}], ",", 
           RowBox[{"ColorFunctionScaling", "\[Rule]", "False"}]}], "]"}]& )], 
     TraditionalForm], TraditionalForm]},
  "Legended",
  DisplayFunction->(GridBox[{{
      TagBox[
       ItemBox[
        PaneBox[
         TagBox[#, "SkipImageSizeLevel"], Alignment -> {Center, Baseline}, 
         BaselinePosition -> Baseline], DefaultBaseStyle -> "Labeled"], 
       "SkipImageSizeLevel"], 
      ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
    GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}}, 
    AutoDelete -> False, GridBoxItemSize -> Automatic, 
    BaselinePosition -> {1, 1}]& ),
  Editable->True,
  InterpretationFunction->(RowBox[{"Legended", "[", 
     RowBox[{#, ",", 
       RowBox[{"Placed", "[", 
         RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]], "Output",
 CellChangeTimes->{{3.864478318195645*^9, 3.864478360310031*^9}, 
   3.864478687692378*^9, 3.864552383101451*^9, {3.864552451462137*^9, 
   3.8645524578885174`*^9}, 3.8645525410542965`*^9, 3.8645526365965147`*^9, 
   3.8645527273738947`*^9, 3.8645527942486963`*^9, 3.8645531417522535`*^9, 
   3.8645533382915287`*^9, 3.86609569664365*^9, 3.86609584584993*^9, 
   3.866096041082574*^9, 3.866096395871772*^9, 3.8660967332652693`*^9, 
   3.8660970471607695`*^9, 3.866097184300061*^9, {3.8660972773998413`*^9, 
   3.866097280970598*^9}, 3.86609731840666*^9, 3.8660977030523944`*^9, 
   3.8679205308900614`*^9, {3.867920651766673*^9, 3.8679206556158466`*^9}, {
   3.8679207889795876`*^9, 3.867920799149659*^9}, 3.8679212300102973`*^9, 
   3.8679213294903817`*^9, {3.867921394244588*^9, 3.8679213994197626`*^9}, {
   3.8679214625761333`*^9, 3.8679214705182734`*^9}, 3.867921692738534*^9, 
   3.8679218437178288`*^9, 3.8679219770477514`*^9, 3.867922082779477*^9, 
   3.8679221342272167`*^9, 3.8679222230260916`*^9, {3.8679223437519436`*^9, 
   3.867922358480153*^9}, 3.867925738101035*^9},
 CellLabel->"Out[94]=",ExpressionUUID->"700747a4-1651-451c-b657-511f9b035916"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", 
   RowBox[{"**", 
    RowBox[{
    "*", " ", "Show", " ", "convergence", " ", "plot", " ", "of", " ", "QBX", 
     " ", "error", " ", "for", " ", "N", " ", "variation"}]}], " ", "****)"}],
   "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"x", "=", 
     RowBox[{"Subdivide", "[", 
      RowBox[{"0", ",", "n", ",", "n"}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"maxN", "[", "nn_", "]"}], ":=", "\[IndentingNewLine]", 
     RowBox[{"Module", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"err", ",", "data"}], "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"err", "=", 
         RowBox[{"DLPERR", "[", "nn", "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"data", "=", 
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Re", "[", 
              RowBox[{"wevalPoints", "[", 
               RowBox[{"[", "idx", "]"}], "]"}], "]"}], ",", 
             RowBox[{"Im", "[", 
              RowBox[{"wevalPoints", "[", 
               RowBox[{"[", "idx", "]"}], "]"}], "]"}], ",", 
             RowBox[{"Abs", "[", 
              RowBox[{"err", "[", 
               RowBox[{"[", "idx", "]"}], "]"}], "]"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"idx", ",", "1", ",", 
             RowBox[{"Length", "[", "err", "]"}]}], "}"}]}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"max", "=", 
         RowBox[{"Max", "[", 
          RowBox[{
           RowBox[{"Select", "[", 
            RowBox[{"data", ",", 
             RowBox[{
              RowBox[{
               RowBox[{"#", "[", 
                RowBox[{"[", "2", "]"}], "]"}], "<", 
               RowBox[{"sc", "[", 
                RowBox[{"#", "[", 
                 RowBox[{"[", "1", "]"}], "]"}], "]"}]}], "&"}]}], "]"}], "[", 
           RowBox[{"[", 
            RowBox[{"All", ",", "3"}], "]"}], "]"}], "]"}]}]}]}], 
      "\[IndentingNewLine]", "]"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"y", "=", 
     RowBox[{"ParallelTable", "[", 
      RowBox[{
       RowBox[{"maxN", "[", "nn", "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"nn", ",", "0", ",", "n"}], "}"}]}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Export", "[", 
     RowBox[{"\"\<data/QBX/DLP/Nconvergence.mat\>\"", ",", "y"}], "]"}], 
    ";"}], "\[IndentingNewLine]", 
   RowBox[{"ListLogPlot", "[", 
    RowBox[{
     RowBox[{"Transpose", "@", 
      RowBox[{"{", 
       RowBox[{"x", ",", "y"}], "}"}]}], ",", 
     RowBox[{"Joined", "\[Rule]", "True"}]}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.862910496525565*^9, 3.862910560713912*^9}, {
   3.8629190232267685`*^9, 3.8629190373020544`*^9}, {3.8629192229405346`*^9, 
   3.862919223306241*^9}, 3.8629197015110397`*^9, {3.862921097076051*^9, 
   3.862921099014985*^9}, {3.8641129673519964`*^9, 3.864112969790104*^9}, {
   3.864318034084178*^9, 3.864318036542841*^9}, {3.866097717948118*^9, 
   3.8660977224479837`*^9}, {3.8679205521680746`*^9, 
   3.8679206111061687`*^9}, {3.8679211393600254`*^9, 3.8679211803011312`*^9}, 
   3.867921326650973*^9},
 CellLabel->"In[95]:=",ExpressionUUID->"c1c7e552-1b5c-48c9-a2e2-9305b8fd50fa"],

Cell[BoxData[
 GraphicsBox[{{}, {{}, {}, 
    {RGBColor[0.368417, 0.506779, 0.709798], PointSize[0.012833333333333334`],
      AbsoluteThickness[1.6], LineBox[CompressedData["
1:eJxTTMoPSmViYGAQBWIQDQMLfB+2ZtUyHIDwPtgvyzFi8jXlh/IZHFy3958V
4BGG8jkcon63W7z6JwrlCzg4HWjQOb1SHMoXcZi1SvrbzFmyUL6Ew28NrrX5
jjJQvozDvBmxH0r2wPgKDjYnb5z5+1UeyldyiC0IT+qOVITyVRyeHqpQmMwN
46s5RCYZcjo8VIDyNRz0MutjZy2FyWs5vN459ZwdrzKUr+Og8XcLz9YCGF/P
4fEytQPL1ytB+QYOHuWOH3WZYPKGDnyrX/HlR6lA+UYOj9Ma3y0sV4XyjR3u
zjl9XMEHJm/iYBYcpOjhrnIAACR8WvU=
      "]]}}, {{}, {}}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->{True, True},
  AxesLabel->{None, None},
  AxesOrigin->{0, -11.412323317181176`},
  DisplayFunction->Identity,
  Frame->{{False, False}, {False, False}},
  FrameLabel->{{None, None}, {None, None}},
  FrameTicks->{{
     Charting`ScaledTicks[{Log, Exp}], 
     Charting`ScaledFrameTicks[{Identity, Identity}]}, {Automatic, Automatic}},
  GridLines->{None, None},
  GridLinesStyle->Directive[
    GrayLevel[0.5, 0.4]],
  Method->{
   "OptimizePlotMarkers" -> True, "OptimizePlotMarkers" -> True, 
    "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
        Identity[
         Part[#, 1]], 
        Exp[
         Part[#, 2]]}& ), "CopiedValueFunction" -> ({
        Identity[
         Part[#, 1]], 
        Exp[
         Part[#, 2]]}& )}},
  PlotRange->{{0, 20.}, {-11.412323317181176`, -2.0612383326894843`}},
  PlotRangeClipping->True,
  PlotRangePadding->{{
     Scaled[0.02], 
     Scaled[0.02]}, {
     Scaled[0.02], 
     Scaled[0.05]}},
  Ticks->FrontEndValueCache[{Automatic, 
     Charting`ScaledTicks[{Log, Exp}]}, {Automatic, {{-11.512925464970229`, 
       FormBox[
        TemplateBox[{"10", 
          RowBox[{"-", "5"}]}, "Superscript", SyntaxForm -> SuperscriptBox], 
        TraditionalForm], {0.01, 0.}}, {-9.210340371976182, 
       FormBox[
        TemplateBox[{"10", 
          RowBox[{"-", "4"}]}, "Superscript", SyntaxForm -> SuperscriptBox], 
        TraditionalForm], {0.01, 0.}}, {-6.907755278982137, 
       FormBox["0.001`", TraditionalForm], {0.01, 0.}}, {-4.605170185988091, 
       FormBox[
        TagBox[
         InterpretationBox[
          StyleBox["\"0.010\"", ShowStringCharacters -> False], 0.01, 
          AutoDelete -> True], NumberForm[#, {
           DirectedInfinity[1], 3}]& ], TraditionalForm], {0.01, 
       0.}}, {-2.3025850929940455`, 
       FormBox[
        TagBox[
         InterpretationBox[
          StyleBox["\"0.100\"", ShowStringCharacters -> False], 0.1, 
          AutoDelete -> True], NumberForm[#, {
           DirectedInfinity[1], 3}]& ], TraditionalForm], {0.01, 
       0.}}, {-13.815510557964274`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-13.122363377404328`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-12.716898269296165`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-12.429216196844383`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-12.206072645530174`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-12.02375108873622, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-11.86960040890896, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-11.736069016284437`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-11.618285980628055`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-10.819778284410283`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-10.41431317630212, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-10.126631103850338`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-9.903487552536127, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-9.721165995742174, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-9.567015315914915, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-9.433483923290392, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-9.315700887634009, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-8.517193191416238, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-8.111728083308073, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-7.824046010856292, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-7.600902459542082, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-7.418580902748128, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-7.264430222920869, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-7.1308988302963465`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-7.013115794639964, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-6.214608098422191, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-5.809142990314028, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-5.521460917862246, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-5.298317366548036, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-5.115995809754082, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-4.961845129926823, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-4.8283137373023015`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-4.710530701645918, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-3.912023005428146, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-3.506557897319982, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-3.2188758248682006`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-2.995732273553991, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-2.8134107167600364`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-2.659260036932778, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-2.5257286443082556`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-2.4079456086518722`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-1.6094379124341003`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-1.2039728043259361`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-0.916290731874155, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-0.6931471805599453, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-0.5108256237659907, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-0.35667494393873245`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-0.2231435513142097, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-0.10536051565782628`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}}, {0., 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}}, {
      0.4054651081081644, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}}, {
      0.6931471805599453, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}}}]]], "Output",
 CellChangeTimes->{
  3.862919704313911*^9, 3.8629200827483573`*^9, 3.8629211400930595`*^9, 
   3.862921408271866*^9, 3.862921814404378*^9, 3.862922292228512*^9, 
   3.863463090631181*^9, 3.864112995806452*^9, 3.8643180177701635`*^9, 
   3.864318058578053*^9, 3.864478370656201*^9, {3.864478458871126*^9, 
   3.8644784887933273`*^9}, {3.864478693954832*^9, 3.8644787049557924`*^9}, 
   3.8645524896510944`*^9, 3.8645526460950413`*^9, 3.864553343961211*^9, 
   3.866095717285291*^9, 3.866095851502528*^9, 3.866096046518077*^9, 
   3.8660964174051466`*^9, 3.8660967543424864`*^9, 3.8660973520390863`*^9, 
   3.8660977469541674`*^9, 3.8679206160865564`*^9, 3.867920801737874*^9, 
   3.867921230659561*^9, {3.867921329753677*^9, 3.867921340548068*^9}, 
   3.867921474360692*^9, 3.8679216946467867`*^9, 3.8679219790045195`*^9, 
   3.867922139830976*^9, 3.867922345612607*^9, 3.8679257399221697`*^9},
 CellLabel->"Out[99]=",ExpressionUUID->"352590dc-6e9e-4cc9-ac30-aff804fcdc58"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", 
   RowBox[{"**", 
    RowBox[{"*", " ", "Export", " ", "remaining", " ", "data"}]}], " ", 
   "****)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"Export", "[", 
     RowBox[{"\"\<data/QBX/DLP/dataxy.mat\>\"", ",", "wevalPoints"}], "]"}], 
    ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Export", "[", 
     RowBox[{"\"\<data/QBX/DLP/error.mat\>\"", ",", 
      RowBox[{"ParallelTable", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Log", "[", 
          RowBox[{"Abs", "[", 
           RowBox[{"DLPERR", "[", "nn", "]"}], "]"}], "]"}], "/", 
         RowBox[{"Log", "[", "10", "]"}]}], ",", 
        RowBox[{"{", 
         RowBox[{"nn", ",", "0", ",", "n"}], "}"}]}], "]"}]}], "]"}], 
    ";"}]}]}]], "Input",
 CellChangeTimes->{{3.862906801656907*^9, 3.862906840886513*^9}, {
   3.8629068779005575`*^9, 3.8629069077735486`*^9}, {3.862906953843136*^9, 
   3.8629069579089127`*^9}, {3.8629078696937017`*^9, 3.8629078854341803`*^9}, 
   3.86290796281295*^9, {3.8629079954333334`*^9, 3.8629080010503225`*^9}, {
   3.862910584410796*^9, 3.8629106385688953`*^9}, {3.8641130129841557`*^9, 
   3.86411301497078*^9}, {3.864318072812992*^9, 3.8643180745320516`*^9}, {
   3.86609775847626*^9, 3.8660977614955645`*^9}, {3.8679208143547397`*^9, 
   3.867920851379735*^9}, {3.867921154760972*^9, 3.867921185775193*^9}},
 CellLabel->
  "In[100]:=",ExpressionUUID->"1475a795-17cc-43aa-90e9-2fba69c76822"],

Cell[BoxData[""], "Input",
 CellChangeTimes->{{3.8629192696990843`*^9, 3.862919275011865*^9}},
 CellLabel->
  "In[102]:=",ExpressionUUID->"c04e5624-3fc1-4cb6-aa99-3ebb8d18582c"]
},
WindowSize->{1904, 999},
WindowMargins->{{-1920, Automatic}, {Automatic, 0}},
TaggingRules->{
 "WelcomeScreenSettings" -> {"FEStarting" -> False}, "TryRealOnly" -> False},
Magnification:>1.2 Inherited,
FrontEndVersion->"12.2 for Microsoft Windows (64-bit) (December 12, 2020)",
StyleDefinitions->"Default.nb",
ExpressionUUID->"ea102d2e-9dee-4b05-8256-86920c6c7617"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[558, 20, 6667, 135, 486, "Input",ExpressionUUID->"f82bcb07-fb1f-46be-9b92-f33ee38db7aa"],
Cell[7228, 157, 2741, 82, 180, "Input",ExpressionUUID->"69b602e8-6ddf-420a-96b8-feceb41bf3cf"],
Cell[CellGroupData[{
Cell[9994, 243, 5797, 142, 300, "Input",ExpressionUUID->"c120534d-ad3d-4b28-a52a-23a177e3226c"],
Cell[15794, 387, 1510, 21, 39, "Output",ExpressionUUID->"580a52f7-9e7e-4cc8-830d-750d6d7a293e"],
Cell[17307, 410, 17451, 300, 288, "Output",ExpressionUUID->"674e5675-b69e-40e3-873d-fad6a4b929c0"]
}, Open  ]],
Cell[34773, 713, 517, 10, 108, "Input",ExpressionUUID->"1eeb517c-5973-4b73-bbc8-23a5703c85c6"],
Cell[35293, 725, 5393, 142, 198, "Input",ExpressionUUID->"95b16ad9-4377-47ec-8371-eab31b1da77b"],
Cell[40689, 869, 5741, 142, 231, "Input",ExpressionUUID->"397765d1-d60b-4c96-b2d7-b1e6cd4d7a1d"],
Cell[46433, 1013, 3214, 80, 156, "Input",ExpressionUUID->"ac8a513c-d4ca-46b4-905a-405de129923f"],
Cell[49650, 1095, 1180, 28, 60, "Input",ExpressionUUID->"376a6ed5-f0d3-4f5b-9257-ec0b082b41d5"],
Cell[CellGroupData[{
Cell[50855, 1127, 8181, 184, 396, "Input",ExpressionUUID->"587aaa57-1a31-458b-9a74-f96f79916698"],
Cell[59039, 1313, 1289, 19, 39, "Output",ExpressionUUID->"e4573d85-d964-488f-85f5-35d7e57333b9"],
Cell[60331, 1334, 48336, 861, 444, "Output",ExpressionUUID->"700747a4-1651-451c-b657-511f9b035916"]
}, Open  ]],
Cell[CellGroupData[{
Cell[108704, 2200, 3300, 83, 276, "Input",ExpressionUUID->"c1c7e552-1b5c-48c9-a2e2-9305b8fd50fa"],
Cell[112007, 2285, 9899, 235, 285, "Output",ExpressionUUID->"352590dc-6e9e-4cc9-ac30-aff804fcdc58"]
}, Open  ]],
Cell[121921, 2523, 1472, 33, 84, "Input",ExpressionUUID->"1475a795-17cc-43aa-90e9-2fba69c76822"],
Cell[123396, 2558, 177, 3, 35, "Input",ExpressionUUID->"c04e5624-3fc1-4cb6-aa99-3ebb8d18582c"]
}
]
*)

