(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 12.2' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       158,          7]
NotebookDataLength[    122668,       2503]
NotebookOptionsPosition[    120619,       2465]
NotebookOutlinePosition[    121148,       2484]
CellTagsIndexPosition[    121105,       2481]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{
Cell[BoxData[
 RowBox[{
  RowBox[{"(*", " ", 
   RowBox[{
   "Classical", " ", "QBX", " ", "method", " ", "solving", " ", "for", " ", 
    "the", " ", "Helmholtz", " ", "equation", " ", "using", " ", "Hankel", 
    " ", "functions", "\[IndentingNewLine]", "An", " ", "attempt", " ", "at", 
    " ", "reproducing", " ", "similar", " ", "results", " ", "found", " ", 
    "in", " ", "paper", " ", "\"\<Quadrature by expansion...\>\"", 
    "\[IndentingNewLine]", "from", " ", "Klockner", " ", "et", " ", 
    RowBox[{"al", "."}]}], " ", "*)"}], "\[IndentingNewLine]", 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"Unprotect", "[", "Power", "]"}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"Power", "[", 
      RowBox[{
       RowBox[{"0", "|", "0."}], ",", 
       RowBox[{"0", "|", "0."}]}], "]"}], "=", "1"}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Protect", "[", "Power", "]"}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"SetDirectory", "[", 
     RowBox[{"NotebookDirectory", "[", "]"}], "]"}], ";"}], 
   "\[IndentingNewLine]", "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{"Prescribe", " ", "conditions", " ", "of", " ", "environment"}], 
    " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"k", "=", "1"}], ";"}], " ", 
   RowBox[{"(*", " ", 
    RowBox[{"wave", " ", "number"}], " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"rho", "[", "s_", "]"}], ":=", "1"}], ";"}], " ", 
   RowBox[{"(*", " ", 
    RowBox[{"Density", " ", "function"}], " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"sc", "[", "s_", "]"}], ":=", 
     RowBox[{
      RowBox[{"s", "^", "2"}], "+", 
      RowBox[{
       RowBox[{"s", "^", "3"}], "/", "10"}], "-", 
      RowBox[{"2", "*", 
       RowBox[{"s", "^", "4"}]}]}]}], ";"}], " ", 
   RowBox[{"(*", " ", 
    RowBox[{"Parameterized", " ", "boundary"}], " ", "*)"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"scp", "[", "s_", "]"}], ":=", 
     RowBox[{
      RowBox[{"sc", "'"}], "[", "s", "]"}]}], ";"}], " ", 
   RowBox[{"(*", " ", 
    RowBox[{"Gradient", " ", "of", " ", "boundary"}], " ", "*)"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"z", "[", "s_", "]"}], ":=", 
     RowBox[{"Sqrt", "[", 
      RowBox[{"1", "+", 
       RowBox[{
        RowBox[{"scp", "[", "s", "]"}], "^", "2"}]}], "]"}]}], ";"}], " ", 
   RowBox[{"(*", " ", 
    RowBox[{"Jacobian", " ", "for", " ", "polar", " ", "transform"}], " ", 
    "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"c", " ", "=", 
     RowBox[{"0", "-", 
      RowBox[{"I", "*", 
       FractionBox["1", "3"]}]}]}], ";"}], " ", 
   RowBox[{"(*", " ", 
    RowBox[{
    "Leaf", " ", "box", " ", "expansion", " ", "center", " ", "for", " ", 
     "demonstration"}], " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"n", "=", " ", "20"}], ";"}], " ", 
   RowBox[{"(*", " ", 
    RowBox[{
    "highest", " ", "number", " ", "of", " ", "expansion", " ", "terms"}], 
    " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"g", "=", " ", "20"}], ";"}], " ", 
   RowBox[{"(*", " ", 
    RowBox[{
    "number", " ", "of", " ", "grid", " ", "points", " ", "along", " ", "x", 
     " ", "and", " ", "y", " ", "axes"}], " ", "*)"}], 
   "\[IndentingNewLine]"}]}]], "Input",
 CellChangeTimes->{{3.824201549034298*^9, 3.8242017306629705`*^9}, 
   3.824201776641529*^9, {3.8242022613614855`*^9, 3.8242022866580477`*^9}, {
   3.8242041709246683`*^9, 3.824204170971409*^9}, {3.8242047032598925`*^9, 
   3.824204711464141*^9}, 3.8242048071184616`*^9, {3.8242059692585754`*^9, 
   3.824205970590394*^9}, {3.82420613069882*^9, 3.8242061353657885`*^9}, {
   3.824206354492281*^9, 3.8242063575644627`*^9}, {3.8242065229371753`*^9, 
   3.824206532063552*^9}, {3.824216395748816*^9, 3.8242164272106795`*^9}, {
   3.824216826990882*^9, 3.824216828866973*^9}, {3.8248106625310774`*^9, 
   3.8248106626601257`*^9}, {3.8248107940125904`*^9, 3.8248107940653563`*^9}, 
   3.8248111836351447`*^9, 3.824815787479703*^9, {3.8248168306747646`*^9, 
   3.82481683078471*^9}, {3.8254080059647636`*^9, 3.8254080061492243`*^9}, {
   3.8260119645833836`*^9, 3.8260119655700293`*^9}, {3.826012169409006*^9, 
   3.8260121810762*^9}, {3.8260126247049007`*^9, 3.826012624801651*^9}, 
   3.8260126634906626`*^9, {3.8260131752408466`*^9, 3.826013189292863*^9}, {
   3.8260136102958045`*^9, 3.826013623485218*^9}, {3.826014012848959*^9, 
   3.82601402435367*^9}, 3.826027053447894*^9, 3.82602727927706*^9, 
   3.826615296514715*^9, 3.826615871082714*^9, {3.826616319064245*^9, 
   3.8266163191172314`*^9}, 3.8266289118377085`*^9, 3.826629262889695*^9, {
   3.8266294841020746`*^9, 3.826629484173549*^9}, {3.859115131535425*^9, 
   3.859115145427741*^9}, {3.859115260951663*^9, 3.8591152622088704`*^9}, {
   3.859895910927072*^9, 3.859895939057004*^9}, 3.859896054134993*^9, 
   3.85989871262799*^9, 3.859899756568331*^9, {3.8629054836069136`*^9, 
   3.862905492965272*^9}, {3.8629059248390064`*^9, 3.8629059259781837`*^9}, 
   3.8629070323324485`*^9, {3.8629087068457103`*^9, 3.862908707059265*^9}, {
   3.862909204798505*^9, 3.862909206503646*^9}, {3.862909765826353*^9, 
   3.862909767981344*^9}, {3.862910654865882*^9, 3.8629106579745955`*^9}, {
   3.8629109193586617`*^9, 3.862910919686694*^9}, {3.862918612462071*^9, 
   3.862918641758218*^9}, 3.862919733771486*^9, 3.862920161891101*^9, 
   3.8629202015304966`*^9, 3.862921154822416*^9, {3.8629220004618025`*^9, 
   3.862922007419752*^9}, {3.8634628746085806`*^9, 3.8634628961666474`*^9}, 
   3.8641107480281377`*^9, {3.864112610767273*^9, 3.8641126395766673`*^9}, {
   3.864113165365574*^9, 3.864113167254612*^9}, 3.864317879485265*^9, {
   3.864477901725404*^9, 3.864477908801912*^9}, {3.8644785551191254`*^9, 
   3.8644785551910267`*^9}, {3.864478603472185*^9, 3.864478604668239*^9}, 
   3.864551995917081*^9, {3.864552037280597*^9, 3.86455211032366*^9}, {
   3.8645522431489735`*^9, 3.8645522457155004`*^9}, 3.8645523049343505`*^9, 
   3.864552662463707*^9, {3.864553273993354*^9, 3.864553274446148*^9}, 
   3.8660954907449923`*^9, 3.8660957839563603`*^9, 3.866095982308855*^9, {
   3.866096180805931*^9, 3.8660962345722947`*^9}, {3.8660968641222615`*^9, 
   3.86609688781585*^9}, 3.866097089007806*^9, {3.8660974909862423`*^9, 
   3.866097509494171*^9}, {3.867920174317848*^9, 3.867920199107699*^9}, {
   3.8679202866043*^9, 3.8679203105027466`*^9}},
 CellLabel->"In[1]:=",ExpressionUUID->"f82bcb07-fb1f-46be-9b92-f33ee38db7aa"],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", 
   RowBox[{"**", 
    RowBox[{
    "*", " ", "Coordinate", " ", "transform", " ", "helper", " ", 
     "functions"}]}], " ", "****)"}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
   "Polar", " ", "coordinate", " ", "pair", " ", "of", " ", "boundary", " ", 
    "point", " ", 
    RowBox[{"w", ".", "r", ".", "t", ".", " ", "c"}]}], " ", "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"rz", "[", "tau_", "]"}], ":=", 
     RowBox[{"Sqrt", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"tau", "-", 
          RowBox[{"Re", "[", "c", "]"}]}], ")"}], "^", "2"}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"sc", "[", "tau", "]"}], "-", 
          RowBox[{"Im", "[", "c", "]"}]}], ")"}], "^", "2"}]}], "]"}]}], 
    ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"argz", "[", "tau_", "]"}], ":=", 
     RowBox[{"Arg", "[", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"tau", "-", 
         RowBox[{"Re", "[", "c", "]"}]}], ")"}], "+", 
       RowBox[{"I", "*", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"sc", "[", "tau", "]"}], " ", "-", " ", 
          RowBox[{"Im", "[", "c", "]"}]}], ")"}]}]}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{"(*", " ", 
    RowBox[{
    "Polar", " ", "coordinate", " ", "pair", " ", "of", " ", "target", " ", 
     "point", " ", 
     RowBox[{"w", ".", "r", ".", "t", ".", " ", "c"}]}], " ", "*)"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"rw", "[", "weval_", "]"}], ":=", 
     RowBox[{"Sqrt", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Re", "[", "weval", "]"}], "-", 
          RowBox[{"Re", "[", "c", "]"}]}], ")"}], "^", "2"}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Im", "[", "weval", "]"}], "-", 
          RowBox[{"Im", "[", "c", "]"}]}], ")"}], "^", "2"}]}], "]"}]}], 
    ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"argw", "[", "weval_", "]"}], ":=", 
     RowBox[{"Arg", "[", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"Re", "[", "weval", "]"}], "-", 
         RowBox[{"Re", "[", "c", "]"}]}], ")"}], "+", 
       RowBox[{"I", "*", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"Im", "[", "weval", "]"}], "-", 
          RowBox[{"Im", "[", "c", "]"}]}], ")"}]}]}], "]"}]}], 
    ";"}]}]}]], "Input",
 CellChangeTimes->{{3.864112227020434*^9, 3.8641122320599575`*^9}, {
  3.867920207461515*^9, 3.867920221657995*^9}},
 CellLabel->"In[13]:=",ExpressionUUID->"69b602e8-6ddf-420a-96b8-feceb41bf3cf"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{"(*", 
   RowBox[{
    RowBox[{
    "**", "**", "**", "**", "**", "**", "**", "**", "**", "**", "**", "**", "**",
      "**", "**", "**", "\[IndentingNewLine]", "Create"}], " ", "mesh", " ", 
    "over", " ", "the", " ", 
    RowBox[{"interval", "\[IndentingNewLine]", "**", "**", "**"}], "*", "*", 
    RowBox[{"**", "**", "**", "**", "**", "**", "**", "**"}]}], "********)"}],
   "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"createMesh", "[", 
      RowBox[{"xlim1_", ",", "xlim2_", ",", "ylim1_", ",", "ylim2_"}], "]"}], 
     ":=", "\[IndentingNewLine]", 
     RowBox[{"tmp", "=", 
      RowBox[{"Select", "[", 
       RowBox[{
        RowBox[{"Flatten", "[", 
         RowBox[{
          RowBox[{"ParallelTable", "[", 
           RowBox[{
            RowBox[{
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{"g", "-", "1", "-", "i"}], ")"}], "/", 
                RowBox[{"(", 
                 RowBox[{"g", "-", "1"}], ")"}]}], ")"}], "*", "xlim1"}], "+", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{"i", "/", 
                RowBox[{"(", 
                 RowBox[{"g", "-", "1"}], ")"}]}], ")"}], "*", "xlim2"}], "+", 
             RowBox[{"I", "*", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{"g", "-", "1", "-", "j"}], ")"}], "/", 
                RowBox[{"(", 
                 RowBox[{"g", "-", "1"}], ")"}]}], ")"}], "*", "ylim1"}], "+", 
             RowBox[{"I", "*", 
              RowBox[{"(", 
               RowBox[{"j", "/", 
                RowBox[{"(", 
                 RowBox[{"g", "-", "1"}], ")"}]}], ")"}], "*", "ylim2"}]}], 
            ",", 
            RowBox[{"{", 
             RowBox[{"i", ",", "0", ",", 
              RowBox[{"g", "-", "1"}]}], "}"}], ",", 
            RowBox[{"{", 
             RowBox[{"j", ",", "0", ",", 
              RowBox[{"g", "-", "1"}]}], "}"}]}], "]"}], ",", "1"}], "]"}], 
        ",", 
        RowBox[{
         RowBox[{
          RowBox[{"Im", "[", "#", "]"}], "\[NotEqual]", 
          RowBox[{"sc", "[", 
           RowBox[{"Re", "[", "#", "]"}], "]"}]}], "&"}]}], "]"}]}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"wevalPoints", "=", 
     RowBox[{"createMesh", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "1"}], "/", "3"}], ",", 
       RowBox[{"1", "/", "3"}], ",", 
       RowBox[{
        RowBox[{"-", "1"}], "/", "2"}], ",", 
       RowBox[{"1", "/", "6"}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{"Length", "[", "wevalPoints", "]"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"p1", "=", 
     RowBox[{"Plot", "[", 
      RowBox[{
       RowBox[{"sc", "[", "s", "]"}], ",", " ", 
       RowBox[{"{", 
        RowBox[{"s", ",", 
         RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", " ", 
       RowBox[{"PlotRange", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"1.1", "*", 
             RowBox[{"Min", "[", 
              RowBox[{"Re", "[", "wevalPoints", "]"}], "]"}]}], ",", 
            RowBox[{"1.1", "*", 
             RowBox[{"Max", "[", 
              RowBox[{"Re", "[", "wevalPoints", "]"}], "]"}]}]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"1.1", "*", 
             RowBox[{"Min", "[", 
              RowBox[{"Im", "[", "wevalPoints", "]"}], "]"}]}], ",", 
            RowBox[{"1.1", "*", 
             RowBox[{"Max", "[", 
              RowBox[{"Im", "[", "wevalPoints", "]"}], "]"}]}]}], "}"}]}], 
         "}"}]}], ",", "  ", 
       RowBox[{"Axes", "\[Rule]", "True"}]}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"p2", "=", 
     RowBox[{"Graphics", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"PointSize", "[", "Large", "]"}], ",", " ", 
        RowBox[{"Point", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Re", "[", "c", "]"}], ",", " ", 
           RowBox[{"Im", "[", "c", "]"}]}], "}"}], "]"}]}], "}"}], "]"}]}], 
    ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"p3", "=", 
     RowBox[{"ListPlot", "[", 
      RowBox[{"Thread", "[", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"Re", "[", "wevalPoints", "]"}], ",", 
         RowBox[{"Im", "[", "wevalPoints", "]"}]}], "}"}], "]"}], "]"}]}], 
    ";"}], "\[IndentingNewLine]", 
   RowBox[{"Show", "[", 
    RowBox[{"p1", ",", "p2", ",", "p3"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.86290552503234*^9, 3.8629055341715975`*^9}, {
   3.862907045492211*^9, 3.86290705499831*^9}, {3.8629071285026073`*^9, 
   3.862907139499606*^9}, {3.8629098157040553`*^9, 3.8629098534925613`*^9}, 
   3.8629186501361322`*^9, {3.8629194231346087`*^9, 3.862919424864961*^9}, {
   3.862920172134658*^9, 3.8629201732778378`*^9}, {3.862920211223621*^9, 
   3.862920221225622*^9}, {3.8629211599615893`*^9, 3.8629211639970884`*^9}, {
   3.862921560685911*^9, 3.8629215718557925`*^9}, {3.8629220209578886`*^9, 
   3.8629220222204413`*^9}, {3.86346290572367*^9, 3.863462908980229*^9}, {
   3.8641107317540913`*^9, 3.8641107320387783`*^9}, {3.8641107750673776`*^9, 
   3.86411077523769*^9}, 3.864112223429037*^9, {3.86431788709599*^9, 
   3.864317902732391*^9}, {3.8644786132297144`*^9, 3.8644786167860384`*^9}, {
   3.864552127698085*^9, 3.8645521342804985`*^9}, {3.8660965472468348`*^9, 
   3.866096555119257*^9}, 3.866096602551182*^9, {3.8660969121915593`*^9, 
   3.86609695016201*^9}, {3.866097514839942*^9, 3.8660975216833096`*^9}, {
   3.8679202345126038`*^9, 3.867920235404002*^9}},
 CellLabel->"In[17]:=",ExpressionUUID->"c120534d-ad3d-4b28-a52a-23a177e3226c"],

Cell[BoxData["400"], "Output",
 CellChangeTimes->{
  3.862905540804943*^9, 3.8629065911462297`*^9, {3.862907113243218*^9, 
   3.8629071403986745`*^9}, 3.8629071782874813`*^9, 3.8629090466608944`*^9, 
   3.8629097800125494`*^9, 3.862909855875724*^9, 3.86291065982838*^9, 
   3.8629109216373515`*^9, 3.862918651682719*^9, 3.862919429545143*^9, 
   3.862919736218258*^9, {3.8629201872455654`*^9, 3.8629202281983323`*^9}, 
   3.862921164861785*^9, 3.862921574940336*^9, 3.862922028002224*^9, 
   3.863462917542016*^9, 3.864110776741166*^9, 3.86411087515793*^9, 
   3.864112283994911*^9, {3.8641126243142643`*^9, 3.8641126423684287`*^9}, 
   3.8641131756218963`*^9, 3.8643179036150265`*^9, 3.8644779923621655`*^9, 
   3.8644781066406484`*^9, 3.86447820511071*^9, 3.86447855881993*^9, 
   3.8644786225375896`*^9, 3.8645521349863997`*^9, 3.8645523949008055`*^9, 
   3.8645525645555964`*^9, 3.864552670722847*^9, 3.864553282683213*^9, 
   3.8660955712229204`*^9, 3.8660957880746145`*^9, 3.8660959864971266`*^9, 
   3.8660962753145733`*^9, {3.866096550855706*^9, 3.866096555844788*^9}, {
   3.8660966036130166`*^9, 3.8660966084338627`*^9}, {3.866096914058324*^9, 
   3.8660969507684507`*^9}, 3.8660970923403473`*^9, 3.866097525672941*^9, 
   3.8679203211905203`*^9, 3.8679256670986614`*^9},
 CellLabel->"Out[19]=",ExpressionUUID->"25ad1e88-a469-43d8-82d5-d1ceee50e4c0"],

Cell[BoxData[
 GraphicsBox[{{{{}, {}, 
     TagBox[
      {RGBColor[0.368417, 0.506779, 0.709798], AbsoluteThickness[1.6], 
       Opacity[1.], LineBox[CompressedData["
1:eJw1mnk4Vd/3xwkZPkIiRCUhpDJlCqtkSgghU0iaJCFE5iHJTAiZ5ylkHi77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        "]]},
      Annotation[#, "Charting`Private`Tag$5703#1"]& ]}, {}}, 
   {PointSize[Large], 
    PointBox[NCache[{0, Rational[-1, 3]}, {0, -0.3333333333333333}]]}, {{}, 
    {RGBColor[0.368417, 0.506779, 0.709798], PointSize[0.007333333333333334], 
     AbsoluteThickness[1.6], PointBox[CompressedData["
1:eJxd2C+o3nUYxuGjrBpmWBXBYLAYNBi8w8IMBhEMGhYEsQgurBgUNFgMKiwJ
BsuCZUGRBWWKDg0KwylDnGAxWC2rbu+76w73KYfP4bx/uODL9/k9j7765ouv
PXhycvLRAycn936/dPj5/drJ4efva/rh2+cuPPLdX+3Hrn7497Pv/dl++tKt
F86f/aP93N3/fvvUrfYrz7/+5KfXf2u/8fiVz75+/9f2O6funL597kb73qvv
nP65ffath/556tKP7XvvdvHM9+0zN1+++wnftM8fXvBl+/DxFy+3D1/vzrvt
w9f575Pow9e5cKX9zAf/PnH5i6vtw9e59W37+Pt6jt/rZv00P81P89P8ND/N
T/PT/DQ/zU/z0/w0P81P89P8ND/NT/M7/v1G/TQ/zU/z0/w0P81P89P8ND/N
T/PT/DQ/zU/z0/w0P81P8zt+j1/qp/lpfpqf5qf5aX6an+an+Wl+mp/mp/lp
fpqf5qf5aX6a3/H/fqqf5qf5aX6an+an+Wl+mp/mp/lpfpqf5qf5aX6an+an
+Wl+x8/9oX6an+an+Wl+mp/mp/lpfpqf5qf5aX6an+an+Wl+mp/mp/kdX3et
fpqf5qf5aX6an+an+Wl+mp/mp/lpfpqf5qf5aX6an+an+R0/56v6aX6an+an
+Wl+mp/mp/lpfpqf5qf5aX6an+an+Wl+mp/md3yfz+un+Wl+mp/mp/lpfpqf
5qf5aX6an+an+Wl+mp/mp/lpfprf8X0/rp/mp/lpfpqf5qf5aX6an+an+Wl+
mp/mp/lpfpqf5qf56fHL+GX8Mn4Zv4xfxi/jl/HL+GX8Mn4Zv4xfxi/jl/HL
+GX8Mn6Z85s5v5nzmzm/mfObOb+Z85s5v5nzWz895zdzfjPnN3N+M+e3fnrO
b+b8Zs5v5vxm7o/M/ZG5PzL3R+b+yNwfmfsjc39k7o/66bk/6qfn/sjcH/XT
c3/UT8/9kbk/MvdHZn7JzC+Z+SUzv2Tml8z8kplfMvNLZn6pn575pX565pfM
/FI/PfNL/fTML5n5JTO/1O/+PFg/PfNzZn7OzM+Z+TkzP2fm58z8XD8983P9
9MzPmfm5fnrm5/rpmZ8z83P99Dy/1U/P81vm+S3z/JZ5fss8v2We3zLPb/XT
8/xWPz3Pb/XT8/xWPz3Pb/XT8/xWPz37g/rp2R9k9geZ/UFmf5DZH2T2B5n9
Qf307A/qp2d/UD89+4P66dkf1E/P/qB+evZX9dOzv8rsrzL7q8z+KrO/yuyv
Mvur+unZX9VPz/6qfnr2V/XTs7+qn579Vf307E/rp2d/mtmfZvanmf1pZn+a
2Z9m9qf107M/rZ+e/Wn9ND89+9P66dmf1k/z0/zu76Prp2d/n9nfZ/b3mf19
Zn+f2d9n9vf107O/r5+e/X39ND89+/v66dnf10/z0/z+B1xqRis=
      "]]}, {{}, {}}}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->{True, True},
  AxesLabel->{None, None},
  AxesOrigin->{0, 0},
  DisplayFunction->Identity,
  Frame->{{False, False}, {False, False}},
  FrameLabel->{{None, None}, {None, None}},
  FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
  GridLines->{None, None},
  GridLinesStyle->Directive[
    GrayLevel[0.5, 0.4]],
  ImagePadding->All,
  Method->{
   "DefaultBoundaryStyle" -> Automatic, 
    "DefaultGraphicsInteraction" -> {
     "Version" -> 1.2, "TrackMousePosition" -> {True, False}, 
      "Effects" -> {
       "Highlight" -> {"ratio" -> 2}, "HighlightPoint" -> {"ratio" -> 2}, 
        "Droplines" -> {
         "freeformCursorMode" -> True, 
          "placement" -> {"x" -> "All", "y" -> "None"}}}}, "DefaultMeshStyle" -> 
    AbsolutePointSize[6], "ScalingFunctions" -> None, 
    "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
        (Identity[#]& )[
         Part[#, 1]], 
        (Identity[#]& )[
         Part[#, 2]]}& ), "CopiedValueFunction" -> ({
        (Identity[#]& )[
         Part[#, 1]], 
        (Identity[#]& )[
         Part[#, 2]]}& )}},
  PlotRange->{{-0.3666666666666667, 0.3666666666666667}, {-0.55, 
   0.18333333333333335`}},
  PlotRangeClipping->True,
  PlotRangePadding->{{0, 0}, {0, 0}},
  Ticks->{Automatic, Automatic}]], "Output",
 CellChangeTimes->{
  3.862905540804943*^9, 3.8629065911462297`*^9, {3.862907113243218*^9, 
   3.8629071403986745`*^9}, 3.8629071782874813`*^9, 3.8629090466608944`*^9, 
   3.8629097800125494`*^9, 3.862909855875724*^9, 3.86291065982838*^9, 
   3.8629109216373515`*^9, 3.862918651682719*^9, 3.862919429545143*^9, 
   3.862919736218258*^9, {3.8629201872455654`*^9, 3.8629202281983323`*^9}, 
   3.862921164861785*^9, 3.862921574940336*^9, 3.862922028002224*^9, 
   3.863462917542016*^9, 3.864110776741166*^9, 3.86411087515793*^9, 
   3.864112283994911*^9, {3.8641126243142643`*^9, 3.8641126423684287`*^9}, 
   3.8641131756218963`*^9, 3.8643179036150265`*^9, 3.8644779923621655`*^9, 
   3.8644781066406484`*^9, 3.86447820511071*^9, 3.86447855881993*^9, 
   3.8644786225375896`*^9, 3.8645521349863997`*^9, 3.8645523949008055`*^9, 
   3.8645525645555964`*^9, 3.864552670722847*^9, 3.864553282683213*^9, 
   3.8660955712229204`*^9, 3.8660957880746145`*^9, 3.8660959864971266`*^9, 
   3.8660962753145733`*^9, {3.866096550855706*^9, 3.866096555844788*^9}, {
   3.8660966036130166`*^9, 3.8660966084338627`*^9}, {3.866096914058324*^9, 
   3.8660969507684507`*^9}, 3.8660970923403473`*^9, 3.866097525672941*^9, 
   3.8679203211905203`*^9, 3.8679256672752185`*^9},
 CellLabel->"Out[23]=",ExpressionUUID->"59cb0c73-c673-4705-a669-bab6b32a264c"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", 
   RowBox[{
    RowBox[{"**", "**", "**", "**", "**", "**", "**", "**", "**"}], "*", " ", 
    "\[IndentingNewLine]", "Single", " ", "Layer", " ", 
    RowBox[{"Potential", "\[IndentingNewLine]", "**", "**", "**"}], "*", "*", 
    RowBox[{"**", "**"}]}], "*******)"}], "\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
    RowBox[{
    "Hankel", " ", "function", " ", "of", " ", "first", " ", "kind", " ", 
     "for", " ", "k"}], "|", 
    RowBox[{"w", "-", "z"}], "|", " ", "argument"}], " ", "*)"}], 
  "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"h", "[", 
      RowBox[{"weval_", ",", "tau_"}], "]"}], ":=", 
     RowBox[{"HankelH1", "[", 
      RowBox[{"0", ",", 
       RowBox[{"k", "*", 
        RowBox[{"Sqrt", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"Re", "[", "weval", "]"}], "-", "tau"}], ")"}], "^", 
           "2"}], "+", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{"Im", "[", "weval", "]"}], "-", 
             RowBox[{"sc", "[", "tau", "]"}]}], ")"}], "^", "2"}]}], 
         "]"}]}]}], "]"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"ESLP", "[", "weval_", "]"}], ":=", 
     RowBox[{
      FractionBox["I", "4"], "*", 
      RowBox[{"NIntegrate", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"h", "[", 
          RowBox[{"weval", ",", "tau"}], "]"}], "*", 
         RowBox[{"rho", "[", "tau", "]"}], "*", 
         RowBox[{"z", "[", "tau", "]"}]}], ",", 
        RowBox[{"{", 
         RowBox[{"tau", ",", 
          RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
        RowBox[{"PrecisionGoal", "\[Rule]", "24"}], ",", 
        RowBox[{"WorkingPrecision", "\[Rule]", "50"}], ",", 
        RowBox[{"MaxRecursion", "\[Rule]", "100"}], ",", 
        RowBox[{"AccuracyGoal", "\[Rule]", "24"}]}], "]"}]}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"slpValues", "=", 
     RowBox[{"ParallelTable", "[", 
      RowBox[{
       RowBox[{"ESLP", "[", 
        RowBox[{"wevalPoints", "[", 
         RowBox[{"[", "idx", "]"}], "]"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"idx", ",", "1", ",", 
         RowBox[{"Length", "[", "wevalPoints", "]"}]}], "}"}]}], "]"}]}], 
    ";"}]}]}]], "Input",
 CellChangeTimes->{{3.824202028932295*^9, 3.82420204680707*^9}, {
   3.8242020887222743`*^9, 3.824202258191167*^9}, {3.824202308786825*^9, 
   3.8242025167851424`*^9}, {3.8242030374626064`*^9, 
   3.8242030377501864`*^9}, {3.824204688243533*^9, 3.824204689309017*^9}, 
   3.8242048122644014`*^9, {3.8242061223262815`*^9, 3.8242061238759737`*^9}, {
   3.8242062474059305`*^9, 3.824206248749133*^9}, {3.824206349380468*^9, 
   3.8242063520995674`*^9}, {3.825418825388903*^9, 3.825418835096904*^9}, {
   3.826011996767233*^9, 3.826012039030652*^9}, {3.862908646761433*^9, 
   3.862908810706462*^9}, {3.864112252990475*^9, 3.864112260226738*^9}, {
   3.8645521455637217`*^9, 3.864552145715685*^9}, 3.8645530655083466`*^9, {
   3.8645532540184193`*^9, 3.8645532547275476`*^9}, {3.8679202568668685`*^9, 
   3.867920258160318*^9}, {3.867921208791154*^9, 3.8679212120115213`*^9}},
 CellLabel->"In[24]:=",ExpressionUUID->"a069f3ac-999e-4c5f-a3bd-770451c52dd9"],

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{"(*", 
   RowBox[{"**", 
    RowBox[{
    "*", " ", "Calculate", " ", "expansion", " ", "coefficients", " ", 
     "about", " ", "c", " ", "using", " ", 
     RowBox[{"Graff", "'"}], "s", " ", "Addition", " ", "Theorem"}]}], " ", 
   "****)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"h2", "[", 
     RowBox[{"tau_", ",", "l_"}], "]"}], ":=", 
    RowBox[{
     RowBox[{"HankelH1", "[", 
      RowBox[{"l", ",", 
       RowBox[{"k", "*", 
        RowBox[{"rz", "[", "tau", "]"}]}]}], "]"}], "*", 
     RowBox[{"E", "^", 
      RowBox[{"(", 
       RowBox[{"I", "*", "l", "*", 
        RowBox[{"argz", "[", "tau", "]"}]}], ")"}]}], "*", 
     RowBox[{"rho", "[", "tau", "]"}], "*", 
     RowBox[{"z", "[", "tau", "]"}]}]}], 
   RowBox[{"(*", " ", "Integrand", " ", "*)"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"slpcoef", "[", "l_", "]"}], ":=", 
    RowBox[{
     FractionBox["I", "4"], "*", 
     RowBox[{"NIntegrate", "[", 
      RowBox[{
       RowBox[{"h2", "[", 
        RowBox[{"tau", ",", "l"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"tau", ",", 
         RowBox[{"-", "1"}], ",", "1"}], "}"}], ",", 
       RowBox[{"PrecisionGoal", "\[Rule]", "24"}], ",", 
       RowBox[{"WorkingPrecision", "\[Rule]", "50"}], ",", 
       RowBox[{"MaxRecursion", "\[Rule]", "100"}], ",", 
       RowBox[{"AccuracyGoal", "\[Rule]", "24"}]}], "]"}]}]}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"slpCoefs", " ", "=", " ", 
     RowBox[{"ParallelTable", "[", 
      RowBox[{
       RowBox[{"slpcoef", "[", "l", "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"l", ",", 
         RowBox[{"-", "n"}], ",", "n"}], "}"}]}], "]"}]}], ";"}]}]}]], "Input",
 CellChangeTimes->{{3.824203520125728*^9, 3.824203540165906*^9}, {
   3.824203570707396*^9, 3.824203620480625*^9}, 3.824203786047014*^9, 
   3.8242038186393237`*^9, {3.8242038487334943`*^9, 3.8242039208214703`*^9}, {
   3.8242046776915874`*^9, 3.824204679394264*^9}, {3.8242047793398705`*^9, 
   3.8242048820237093`*^9}, {3.8242049168335686`*^9, 3.824205012085031*^9}, {
   3.8242051177196884`*^9, 3.824205130141483*^9}, {3.824205481375419*^9, 
   3.824205499633629*^9}, {3.82420592029849*^9, 3.8242059249972258`*^9}, {
   3.8242059782133737`*^9, 3.8242059783629704`*^9}, {3.824206366227577*^9, 
   3.8242063918481092`*^9}, 3.82420658938365*^9, {3.824206799648309*^9, 
   3.824206799696867*^9}, {3.824206893792466*^9, 3.824206939889509*^9}, {
   3.8242069814542*^9, 3.824207018713192*^9}, {3.824207513624135*^9, 
   3.824207541408094*^9}, {3.8248056861756754`*^9, 3.824805689036023*^9}, 
   3.8248107138907228`*^9, 3.8254067687513146`*^9, {3.8254080450172415`*^9, 
   3.8254080605284595`*^9}, {3.8254080959310703`*^9, 
   3.8254081314952955`*^9}, {3.825408371541501*^9, 3.825408399604638*^9}, {
   3.8254084690959454`*^9, 3.8254084712704053`*^9}, {3.8260120660150075`*^9, 
   3.826012082003981*^9}, {3.826012158060155*^9, 3.826012166346197*^9}, {
   3.8260125790460396`*^9, 3.8260125927339067`*^9}, 3.8260132235777283`*^9, 
   3.8260135400386667`*^9, {3.826014292630191*^9, 3.8260142946195307`*^9}, {
   3.8260143578656497`*^9, 3.8260143875906897`*^9}, {3.826614016393237*^9, 
   3.8266140176847973`*^9}, {3.862905936743065*^9, 3.862905938890128*^9}, {
   3.8629089345458546`*^9, 3.8629089746686125`*^9}, {3.8645521743289595`*^9, 
   3.86455217489744*^9}, 3.864553075681569*^9, {3.864553257132104*^9, 
   3.8645532575709257`*^9}, {3.8679202706195307`*^9, 3.867920272318844*^9}, {
   3.867920341850747*^9, 3.867920343006847*^9}, {3.8679223004615083`*^9, 
   3.867922300645939*^9}},
 CellLabel->"In[27]:=",ExpressionUUID->"212898ec-88d3-4f23-97a4-a339869bc2ba"],

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{"(*", 
   RowBox[{"**", 
    RowBox[{
    "*", " ", "Calculate", " ", "the", " ", "approximate", " ", "numerical", 
     " ", "single", " ", "layer", " ", "solution", " ", "using", " ", 
     "exponential", " ", "expansion", " ", "coefficients"}]}], " ", "****)"}],
   "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"NSLPpart", "[", 
      RowBox[{"idx_", ",", "nn_"}], "]"}], ":=", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"nn", "\[Equal]", "0"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"slpCoefs", "[", 
         RowBox[{"[", 
          RowBox[{"n", "+", "1"}], "]"}], "]"}], "*", 
        RowBox[{"BesselJ", "[", 
         RowBox[{"0", ",", 
          RowBox[{"k", "*", 
           RowBox[{"rw", "[", 
            RowBox[{"wevalPoints", "[", 
             RowBox[{"[", "idx", "]"}], "]"}], "]"}]}]}], "]"}], "*", 
        RowBox[{"E", "^", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", "I"}], "*", "0", "*", 
           RowBox[{"argw", "[", 
            RowBox[{"wevalPoints", "[", 
             RowBox[{"[", "idx", "]"}], "]"}], "]"}]}], ")"}]}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"Sum", "[", 
        RowBox[{
         RowBox[{
          RowBox[{"slpCoefs", "[", 
           RowBox[{"[", 
            RowBox[{"l", "+", "n", "+", "1"}], "]"}], "]"}], "*", 
          RowBox[{"BesselJ", "[", 
           RowBox[{"l", ",", 
            RowBox[{"k", "*", 
             RowBox[{"rw", "[", 
              RowBox[{"wevalPoints", "[", 
               RowBox[{"[", "idx", "]"}], "]"}], "]"}]}]}], "]"}], "*", 
          RowBox[{"E", "^", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"-", "I"}], "*", "l", "*", 
             RowBox[{"argw", "[", 
              RowBox[{"wevalPoints", "[", 
               RowBox[{"[", "idx", "]"}], "]"}], "]"}]}], ")"}]}]}], ",", 
         RowBox[{"{", 
          RowBox[{"l", ",", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"-", "nn"}], ",", "nn"}], "}"}]}], "}"}]}], "]"}]}], 
      "]"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"nslpValues", "=", 
     RowBox[{"ParallelTable", "[", 
      RowBox[{
       RowBox[{"NSLPpart", "[", 
        RowBox[{"idx", ",", "nn"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"idx", ",", "1", ",", 
         RowBox[{"Length", "[", "wevalPoints", "]"}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"nn", ",", "0", ",", "n"}], "}"}]}], "]"}]}], 
    ";"}]}]}]], "Input",
 CellChangeTimes->{{3.8242077258502274`*^9, 3.824207880985754*^9}, {
   3.8248085384828787`*^9, 3.8248085461014433`*^9}, {3.824811119025569*^9, 
   3.824811126453607*^9}, {3.8254066173005476`*^9, 3.825406627116387*^9}, {
   3.825418719619585*^9, 3.8254187247704167`*^9}, {3.825418857050461*^9, 
   3.825418858819172*^9}, 3.8260123457232413`*^9, 3.8260143722895737`*^9, {
   3.826616390048973*^9, 3.826616391342514*^9}, {3.8266290082780876`*^9, 
   3.8266290084941325`*^9}, {3.862905911881487*^9, 3.862905919568202*^9}, {
   3.8629090129256096`*^9, 3.862909040797291*^9}, {3.86290933681627*^9, 
   3.8629095577602997`*^9}, {3.8629097505041924`*^9, 
   3.8629097587316904`*^9}, {3.862910809891706*^9, 3.862910810010892*^9}, {
   3.8645521887392616`*^9, 3.864552221624096*^9}, {3.8645522585816827`*^9, 
   3.864552295200273*^9}, {3.8645525241428895`*^9, 3.8645525277206726`*^9}, {
   3.8645530803643336`*^9, 3.8645530823388042`*^9}, {3.864553251147619*^9, 
   3.864553262683391*^9}, {3.867920371655755*^9, 3.8679203988002663`*^9}, {
   3.867920442599992*^9, 3.8679204742384305`*^9}},
 CellLabel->"In[30]:=",ExpressionUUID->"c5efc32b-2405-4205-9699-e1dd2563ce85"],

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{"(*", " ", 
   RowBox[{
   "Create", " ", "an", " ", "error", " ", "function", " ", "to", " ", 
    "compare", " ", "the", " ", "exact", " ", "solution", " ", "to", " ", 
    "the", " ", "approximate", " ", "numerical", " ", "solution"}], " ", 
   "*)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"SLPERR", "[", "Nmax_", "]"}], ":=", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"slpValues", "-", 
       RowBox[{"Sum", "[", 
        RowBox[{
         RowBox[{"nslpValues", "[", 
          RowBox[{"[", 
           RowBox[{"All", ",", "nn"}], "]"}], "]"}], ",", 
         RowBox[{"{", 
          RowBox[{"nn", ",", "1", ",", 
           RowBox[{"Nmax", "+", "1"}]}], "}"}]}], "]"}]}], ")"}], "/", 
     "slpValues"}]}], ";"}]}]], "Input",
 CellChangeTimes->{{3.8242079025167246`*^9, 3.8242079768914595`*^9}, 
   3.8260144014067483`*^9, {3.8629059492370787`*^9, 3.8629059567195883`*^9}, {
   3.862907916912027*^9, 3.862907948098891*^9}, {3.8629080840428677`*^9, 
   3.8629080932208815`*^9}, {3.8629086213767204`*^9, 
   3.8629086249588957`*^9}, {3.8629099238468246`*^9, 3.862910051851651*^9}, {
   3.862910086273987*^9, 3.8629100875023394`*^9}, {3.8629101606495447`*^9, 
   3.862910167828006*^9}, 3.8660955867569017`*^9, {3.866095677881569*^9, 
   3.8660956908723307`*^9}, 3.866097236336748*^9, {3.8679204840533433`*^9, 
   3.8679204944883223`*^9}, {3.867920779831871*^9, 3.867920780898728*^9}},
 CellLabel->"In[32]:=",ExpressionUUID->"f565b706-637e-4ec3-a5df-dd05b33bab01"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[IndentingNewLine]", 
  RowBox[{"(*", 
   RowBox[{"**", 
    RowBox[{
    "*", " ", "Create", " ", "a", " ", "graphical", " ", "representation", 
     " ", "of", " ", "the", " ", "error", " ", "for", " ", "choice", " ", 
     "of", " ", "N"}]}], " ", "****)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"err", "=", 
     RowBox[{"SLPERR", "[", "n", "]"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"data", "=", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         RowBox[{"Re", "[", 
          RowBox[{"wevalPoints", "[", 
           RowBox[{"[", "idx", "]"}], "]"}], "]"}], ",", 
         RowBox[{"Im", "[", 
          RowBox[{"wevalPoints", "[", 
           RowBox[{"[", "idx", "]"}], "]"}], "]"}], ",", 
         RowBox[{
          RowBox[{"Log", "[", 
           RowBox[{"Abs", "[", 
            RowBox[{"err", "[", 
             RowBox[{"[", "idx", "]"}], "]"}], "]"}], "]"}], "/", 
          RowBox[{"Log", "[", "10", "]"}]}]}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"idx", ",", "1", ",", 
         RowBox[{"Length", "[", "err", "]"}]}], "}"}]}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"data", "=", 
     RowBox[{"data", "/.", 
      RowBox[{"Indeterminate", "\[Rule]", 
       RowBox[{"-", "16"}]}]}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{"maxError", "=", 
    RowBox[{"Max", "[", 
     RowBox[{
      RowBox[{"Select", "[", 
       RowBox[{"data", ",", 
        RowBox[{
         RowBox[{
          RowBox[{"#", "[", 
           RowBox[{"[", "2", "]"}], "]"}], "<", 
          RowBox[{"sc", "[", 
           RowBox[{"#", "[", 
            RowBox[{"[", "1", "]"}], "]"}], "]"}]}], "&"}]}], "]"}], "[", 
      RowBox[{"[", 
       RowBox[{"All", ",", "3"}], "]"}], "]"}], "]"}]}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"minError", "=", 
     RowBox[{"Min", "[", 
      RowBox[{
       RowBox[{"Select", "[", 
        RowBox[{"data", ",", 
         RowBox[{
          RowBox[{
           RowBox[{"#", "[", 
            RowBox[{"[", "2", "]"}], "]"}], "<", 
           RowBox[{"sc", "[", 
            RowBox[{"#", "[", 
             RowBox[{"[", "1", "]"}], "]"}], "]"}]}], "&"}]}], "]"}], "[", 
       RowBox[{"[", 
        RowBox[{"All", ",", "3"}], "]"}], "]"}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"plot1", "=", 
     RowBox[{"ListDensityPlot", "[", 
      RowBox[{"data", ",", "\[IndentingNewLine]", 
       RowBox[{"RegionFunction", "\[Rule]", 
        RowBox[{"Function", "[", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{"x", ",", "y", ",", "z"}], "}"}], ",", 
          RowBox[{"y", "<", 
           RowBox[{"sc", "[", "x", "]"}]}]}], "]"}]}], ",", 
       "\[IndentingNewLine]", " ", 
       RowBox[{"PlotRange", "\[Rule]", 
        RowBox[{"{", 
         RowBox[{
          RowBox[{"{", 
           RowBox[{
            RowBox[{"1.1", "*", 
             RowBox[{"Min", "[", 
              RowBox[{"Re", "[", "wevalPoints", "]"}], "]"}]}], ",", 
            RowBox[{"1.1", "*", 
             RowBox[{"Max", "[", 
              RowBox[{"Re", "[", "wevalPoints", "]"}], "]"}]}]}], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{
            RowBox[{"1.1", "*", 
             RowBox[{"Min", "[", 
              RowBox[{"Im", "[", "wevalPoints", "]"}], "]"}]}], ",", 
            RowBox[{"1.1", "*", 
             RowBox[{"Max", "[", 
              RowBox[{"Im", "[", "wevalPoints", "]"}], "]"}]}]}], "}"}], ",", 
          "All"}], "}"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"ColorFunctionScaling", "\[Rule]", "False"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"ColorFunction", "\[Rule]", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{"ColorData", "[", "\"\<DarkRainbow\>\"", "]"}], "[", 
           RowBox[{"Rescale", "[", 
            RowBox[{"#", ",", " ", 
             RowBox[{"{", 
              RowBox[{"minError", ",", "maxError"}], "}"}]}], "]"}], "]"}], 
          " ", "&"}], ")"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"PlotLegends", "\[Rule]", 
        RowBox[{"BarLegend", "[", 
         RowBox[{"{", 
          RowBox[{"Automatic", ",", 
           RowBox[{"{", 
            RowBox[{"minError", ",", "maxError"}], "}"}]}], "}"}], "]"}]}]}], 
      "]"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"plot2", "=", 
     RowBox[{"Graphics", "[", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"PointSize", "[", "Large", "]"}], ",", " ", 
        RowBox[{"Point", "[", 
         RowBox[{"{", 
          RowBox[{
           RowBox[{"Re", "[", "c", "]"}], ",", " ", 
           RowBox[{"Im", "[", "c", "]"}]}], "}"}], "]"}]}], "}"}], "]"}]}], 
    ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"plot3", "=", 
     RowBox[{"Plot", "[", 
      RowBox[{
       RowBox[{"sc", "[", "x", "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"x", ",", 
         RowBox[{"-", "1"}], ",", "1"}], "}"}]}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{"Show", "[", 
    RowBox[{"plot1", ",", "plot2", ",", "plot3"}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.8260175730494905`*^9, 3.8260175875731335`*^9}, {
   3.826017725764667*^9, 3.826017738986579*^9}, {3.8260178523475857`*^9, 
   3.8260179248973093`*^9}, {3.826018581433931*^9, 3.8260185838928394`*^9}, {
   3.8266278422806926`*^9, 3.826627889655772*^9}, {3.842620189256916*^9, 
   3.842620192955665*^9}, {3.8426202463370495`*^9, 3.8426202633717985`*^9}, {
   3.842621023392301*^9, 3.842621027180705*^9}, {3.8591154732117615`*^9, 
   3.859115484974616*^9}, {3.859115710923396*^9, 3.859115726224893*^9}, {
   3.8591157937145205`*^9, 3.85911582271305*^9}, {3.859115888253096*^9, 
   3.859115924136121*^9}, {3.859896095794634*^9, 3.859896096879325*^9}, {
   3.8598961295913277`*^9, 3.859896139559779*^9}, {3.859897978624682*^9, 
   3.859898023988381*^9}, {3.859898652079686*^9, 3.8598986524523697`*^9}, {
   3.8629057254688616`*^9, 3.8629057266622734`*^9}, {3.862908066659073*^9, 
   3.8629080706782465`*^9}, 3.862910264320558*^9, {3.8629103081895604`*^9, 
   3.8629103820593705`*^9}, {3.8629104487036076`*^9, 3.862910449055311*^9}, {
   3.8629108535404935`*^9, 3.8629109034562116`*^9}, {3.862911076883559*^9, 
   3.862911082335763*^9}, {3.862911334898176*^9, 3.8629113364819202`*^9}, {
   3.862918941500255*^9, 3.862918941822801*^9}, {3.8629190836532764`*^9, 
   3.8629191141001663`*^9}, {3.862919337703516*^9, 3.8629193819062657`*^9}, {
   3.862919478060832*^9, 3.8629194788278165`*^9}, {3.862920771338073*^9, 
   3.862920806149024*^9}, {3.8629209575855923`*^9, 3.862920961089507*^9}, {
   3.862921011200637*^9, 3.862921013077097*^9}, {3.8629210458460107`*^9, 
   3.8629210654203663`*^9}, {3.8634630263533435`*^9, 
   3.8634630569106517`*^9}, {3.8641120891408486`*^9, 3.8641120917617383`*^9}, 
   3.8641131930125256`*^9, {3.864113405496626*^9, 3.8641134159438467`*^9}, {
   3.8641134854182467`*^9, 3.864113492501912*^9}, {3.864113546514684*^9, 
   3.8641135466523128`*^9}, {3.8643179967892838`*^9, 3.864317996894004*^9}, {
   3.864478052227618*^9, 3.86447808742808*^9}, {3.864478160442616*^9, 
   3.864478171594324*^9}, {3.8644782181334357`*^9, 3.864478221118528*^9}, {
   3.8644782608965416`*^9, 3.864478272498558*^9}, {3.8644783049774323`*^9, 
   3.8644783592340837`*^9}, {3.864552348121228*^9, 3.8645523574758296`*^9}, {
   3.8645524471703196`*^9, 3.864552455682497*^9}, {3.866097280102521*^9, 
   3.8660972804679413`*^9}, {3.866097316335229*^9, 3.866097316598853*^9}, {
   3.8660975507777195`*^9, 3.8660975577554092`*^9}, {3.8679205062393017`*^9, 
   3.867920528281221*^9}, {3.867920649736471*^9, 3.867920654720306*^9}, {
   3.867920792729556*^9, 3.8679207986355762`*^9}, {3.867921227481966*^9, 
   3.8679212355508432`*^9}, {3.867924653097847*^9, 3.867924670255333*^9}},
 CellLabel->"In[33]:=",ExpressionUUID->"587aaa57-1a31-458b-9a74-f96f79916698"],

Cell[BoxData[
 RowBox[{"-", 
  "3.0920056331802952144583942678939847344079447595005048658588`47.\
45151461143642"}]], "Output",
 CellChangeTimes->{{3.864478318195645*^9, 3.864478360310031*^9}, 
   3.864478687692378*^9, 3.864552383101451*^9, {3.864552451462137*^9, 
   3.8645524578885174`*^9}, 3.8645525410542965`*^9, 3.8645526365965147`*^9, 
   3.8645527273738947`*^9, 3.8645527942486963`*^9, 3.8645531417522535`*^9, 
   3.8645533382915287`*^9, 3.86609569664365*^9, 3.86609584584993*^9, 
   3.866096041082574*^9, 3.866096395871772*^9, 3.8660967332652693`*^9, 
   3.8660970471607695`*^9, 3.866097184300061*^9, {3.8660972773998413`*^9, 
   3.866097280970598*^9}, 3.86609731840666*^9, 3.8660977030523944`*^9, 
   3.8679205308900614`*^9, {3.867920651766673*^9, 3.8679206556158466`*^9}, {
   3.8679207889795876`*^9, 3.867920799149659*^9}, 3.8679256871129727`*^9},
 CellLabel->"Out[36]=",ExpressionUUID->"e727b0a1-d7be-47f9-97ca-3912a7e7b333"],

Cell[BoxData[
 TemplateBox[{
   GraphicsBox[{
     GraphicsComplexBox[CompressedData["
1:eJx1mHtsnmUVwBtBMokaYXEjQwUyTcQsyMQrt4OiTBGxmxcYIY1OKerAsrgg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      "], {{{
         EdgeForm[], 
         GrayLevel[0.8], 
         GraphicsGroupBox[{
           PolygonBox[CompressedData["
1:eJxNmXWYltUWxed8MNTQMRJDDDDAkI50dwsiIV0KQ4OECBKXlBJJAy8qoaSI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            "]], 
           PolygonBox[CompressedData["
1:eJw11OdvT3EUx/Hv96qSWA0h8ZCH/gXa0BpN/Ai1N61Ro/3Ze8V6JEbsUUL0
Aam9Q4wQo3ZDaGxiBUFSUqLifXI+ffDKub9773ec8z33164wXVCahBBSyMCm
GMJXjMYafi8iLkRfFOAOlmEE+mAHDmIwpuIEWjHhAmJvbMAndMMS5vsSfQ2b
ewDxHu5qrvbc+03cjnPB53yG5chGjp79QZHe+U7cgjKuBxJ34h/GopJ7r4kr
0QW5OK81ba1s5bIOk5TTIxxFCr3QkFxWE3sgH1vxC4WoYI49lgsmYA4OIzBm
OrE7fmqMvTuIWKc92t6GEPfhOIZjvnL5gTHK6aNqaLXrT/yAVeiqmu5GY9Yb
RuyJS7iPfsizOXAo+lrTUI21KNaalXiCtuiAC3iJzpiMKjyIXlvbz3riC/WC
vfNcZ5SjHlmhOWzsSNTgivK3mj1FOTqp5la7G5itGj7EEdV6Jh7jmM5iLi7i
ra0f/EytFu8wRTVZjFMYpRostdphaPQavYneE7l65zLeq1Y2xzdsRqnO4CRe
KRdb8zNxI0qifyONEj8Dq31t8Fyacm+8crK1T2Oc9nAVCc+rg/eI9VYDfs+I
3mPXkJF479i9WmxDOnrP29iy+rMMvpf90XujRLWu0llZzW1sRfTeSyu3A9G/
PcuxCfOd0d5tz3+jf9P2Lds3lMnzXToLO5PWifes9WpHnEUL7s1TDzXnulni
30JKuV/HLNXgluawsfYfkcX1Xp2tnXHLxHvSerFYY20P+ZrjpsbYu7ZmG65v
R/+vmYj/hLOeCA==
            "]]}]}, {}, {}, {}, {}}}, VertexColors -> CompressedData["
1:eJztefk3lW//vWhSIUIUZYhKRJFQbKIMSUpIUsYSDYqkSMlQlESigQqVhCia
EFFEdMZbHOPB4UwplaFS+t7PH/D94fmsz7Oe9/qsfjlnrb2udZ/rvl77tV97
X0fV+9BWP1ERERF58kOM/N4Rsm69YzYXUnq123+uoCO5bKhHI54CmO8sK+Vx
sc7QSFN2FhMrz0hPtj5FxaHlr79evcHD8q0NxKdXLdjZFh7j850G8XvO86jm
PBR+muEX6tEMnRivlAduNJSfd9GMOMzDxPQLX+zvfsAylh3l9UMa6E925dEX
8DEhbhD3wqAdZk6ZCU2ddGyg7W3csImH6cmtjQcfNcMwVfp06CEaCm7pX0xe
woPVoKGLfhmBadKZN211aZC0LW60ucnD1tajSdOYLdDbMiAp/5uGaNGJ5sZL
XMhM1dD5U05DSbwcr38jBV7dYtfKyPc1ecQOqzOgQ3tr2f1rCRTQbiRsU3Dn
YtmPVo/bdyjgezPtVC+9/6/hbrrNd0aMeHjrFR16dUkz5Oi7Tn/ZSEM9O69v
niUfKopt0cpLO9A6g5u2VpmB/m0HnyWO8WAXxD5vpd2GfQd2Sy2+RkdbPWPn
9Tl8MEObRXT5bagiHlbqvaNjmTqrQGwnH0LjRb09tA7klJzl19swsKXQc7tE
oQB6NUbxj2p78fm5evSLASYSHTRjjSz4kAl3Kt2v1IGwafHWXxQZMDrvH0Xh
8tD5yK5vVQ0Ls/tiD9w6SUftF68fE408zHTOnsmdysLZZxFpIpvoMB9o9pC9
zENmgvOuS2dbUB3nb32KT8N3Cv3e3jYuaM4lPjX5DDhIpQ+t30SFgZvOuLoN
D45Z+71aLjbj0+PZ7eF7aP+1uhj21rlTTvLh5lxlcLy0E4py6zI7ohkQtT6i
ODeCD0HN4irB/U40+MgVdJwmz7N+dJ1FOR9bb/hfd9najcvoPmD3mQGvrF/u
x6UFeJ4Z/3mkgQ2VIDnxe6FMzAl0Spy5XQB1CUW/9cU92BwVfzcij4kHOtON
U2sEWEw3Dfw6sw/n5USOrRllQsbRT0gNFEDs4rep9UM9GMmZkCx7xsQeSsCp
7kE+9N6d6bvtxIau/2hPvTUT0Wryz+hH+GCnt1UcD+iE8dFZHrlBDLSeTaS5
a/Px65ZhiCCpHduiV28KG6Hj4kW7u/soPHw9J9+lM5eFiAmanskWOh5mql/Q
F+NB11JM/2wlE4a3ZXcL28l6jdl2eTdxoZbz/azrJrKOqyJzLJSo/2vnr2Jo
sgHkeboVixrZkuc5czx0eAN5nlZB+3KeGgqw8dTkqZGWPZCZb/7geDITsjb7
Ize/FOCg9FLqq5+9SJDd13HoKxOnxH3SQw2FYKpEO6vHcXBGJv/BU1sCCukx
VvcPCmFWesvlrGk/Sn6+7FtxhEB6W9N4gI8Q57RPmrMU+qEfaKuuGEDAWKnG
tQxCDM6zuL77DgeVcxNNfRwJLLf3U2lbLcS+mvKs/AQOmLVDDU/tCFgfXbfT
4a4ABwj7yQEFvbCnThbR6GGir7LGmNnFh+60UI1hFTa+mbnLndBh4tModb+p
NR+PnNZ0j1p14GWjyYpjixh4Fls4X2ISH1UqzaZr97fB4+73t9V5dARKrxGp
E+HB52qMmM5DJjYG/VA6yKTC3q51D2K4cLnsJRO+lwZ5M1bsFJV/X/cqj459
by4RwOdamVRzey/EYnclZgiZMNmlOUlWVwjzDTfcxQ9ysPr94We+VgTkjzc3
zDwjxGLWc977iH7MNmafW3qaQOJKzaqaj0JsPr1jQpo1AMnFNx+Lv//XesuS
B7lCfE3tVU7IInn0Pu7xz+sElF94NwcNCvF8kX5yRjgXrMPfSgPJ9XuuzPps
zRMi2z7KNfjdAHwU5BOpDQQiRD1P740X4vxmejA9tR9TRQ6fC40lsJ6dfzRa
Tghp+s05N+ZxoHl5pcrPFQRqT2oliuULIBqx+t6Mql48ryi5adHPROZ2Fd+n
DgKsTPDZTEvugUX7lHOvs5nYNj7q07OKD+/i0aaZtHZEGGU0K09loO4sv/j+
v+ZalOWVu/rtIBQm+DHkXNubtCtO+wMXWvtq/C2TGJgibfDqhNm/3xcqE+Xz
r08XonT10fFjQ32wPWkoUriEQEN7YkYMVYiwflNq9vEByPkVNX1/RiCt/URJ
1lMhUuZH3Nt7jYuDd16YNN0lcDFhmUB3sxDbU6QfZ9O5+F2wqFXMnUCscic1
rluAlGJ7aWdJHrLjowV2Mwj8TE65XEAXgFN94WWHHA9nnK7DWoRAqv+8kcEF
QviddlboEnLxOHxHi4oRAd/w/a7eMkI0r2vvLvrKRW6/oayULoHttg9mz0kS
olB20JH3kIu0ZWEB+vEE8jQDK45eFuKgcfzZ7Cf9YL8a/l56gcDLoM8iUy4I
oFRrI8Vx68UKSZ1HgxQmcudVrrgsJyD1aJLi9hY2RD/O3Xr9OBNmTfnp3F88
xFBiaA6ObUgUGxuLyqZjXrD7jYf9PCTb+eUPlLDQ5WR1gXecjvJ7e8qMLnPR
8uTM8LYmGn4+imV7bqPgaZWvs91jIa5E3TU0nzeAhS1yDwXZBKbMusZ+kieE
g/SUsgO3uVhzO1wQkkHAUKsyi0YVYMPKhyHd8jws9BJof55gYtIc56Dq+3y0
TIgK9bbzsM1BRfXFeiZigy9NCkvgwS/SXunpOy6e5e69fpHCwCdZD9vHmjzc
qrrx8F/16i299WnqXQYifhVNd63i4WDM+7EPHC7sg8/EXuIw8GFnzsxxUv8n
OWktXET6JvmODcKap0x0xzQ8ey8txBKngTyRb1zodyRn5ywncHjxgFat5EfU
jC/IkLLhwu/jh/z4fgKZ9XWzuxWEkDTsCLykwcEV7aDdfgYERopHtqhXCeDV
vjvlmUgfdN6bsV99Y+LnPNOzrmF8tOsdGcxP6kRHx2+5zScYWDTs16tEznd5
nfyKBnK+O8plLrtIzvfw3c8fSSzkYS3xJF75BIHrwWlyuQtoaHidVZzGJfk5
8vDTu0gukvwq7+uR/RsbKnrgVKsAEoEdyxfL8DD7ZtpcwykEbN78WfV2Gh/5
p+3scqbzQFyYFfFMlIk1b9zHfin1I/Bm8WQZxQH0fL3U4+BFh+3GuUpSXDb6
NgXT7jX24bX12/fhoMGg+7K/v2IvWpduWf47jAO7qCHXhCga1l6oM9nT3Ipb
D7Mrh/V74PtLckyTRoVNWYm5+RIuPu1QNsibz0W3i9Ve5TUM0IjB4NxVPIg1
moo1kXwYr+ZSzUn/knz/ZwhjpxA7xoc2GpP1bXH4gm5fAq56UZ5XrwrR8cqP
41vXj43uc6f4XyJwjaMa/qVdgGkDy51e7OrDo/4ZSlHTCVDYl+adI32CfseY
hOY7Nk49XXO/mPQJ767FeGWzeLg2hylV482CiUTiqVZfOrL7LVPSx7jo8Pe9
4RLEhPMiQbVTCRVy3yqtCsPJvmM65wS/4EL7U+IZnXACpQPx/lERPCxpeXqF
+pr0y0ZviirrGRBpX5G453U/OEGmppsDB2Al87zg8j06JnSOOUU5d2OreIHC
Q4U+OLM5+kmjVHjqXdwlJvMBsqfo2bdc2GjUXHIqO4kG5WkXXnw+xiB933uT
Lq0uLD660UhBkwFrjZLoxlNtaBxyer7rRQ9qK5a90ZxKxa/Gj0kfi/ugD4ek
5N39uOw4a8c+Fg3uVW6CdzWkz1xYVHt+gIsnHzYdEhlgIPTXzEiHt6Q+HIkM
OTWfh0cZBxLqvzMhb9h8wPO7EHOcswp/TgzAJbmUs7eZQIlIaCJTTYhjmTFh
Ehs46LHKsUxeQ2DmOzNllSIBljc+fHq0qRdPReu6VvOYMLY0DNkFPjqjGmnf
pDqwZZZZRKk8A26sZQ0L9HjQ013eITVIgKYxef/JtTQEeXotbfwtwIyuxJU9
v7lYJGW5V1uFAENEbu5Mez56s7/n79biocE3rlZ1IRM5a2eoDo/3QYQufnZK
Qj+m3Zs5KZ3sF4Z9ccpYSCta3jxPr5frwS9f5WkfflMxXrg1bF4JDSmNKfKv
g1lYlTJ4IdWkA+tOJCipHaRg5Ns9Nbm6ZgQtonk67e3F0qV6qit0GHB/dVXE
6EonJu4TFdW7mfg1cnGV+XA7Mm68COCo92LXBbOPv9ZQ8bbwxFb6Gi7UxcWy
tmlwoZxlELDOkoFdiRoFKQ8EkL3F+aa8jId9a1sYohwmAq9bXl3KEsLVKuZQ
dhyp8z+zGDWVpP6/TXnde0iIj2r7SuMt+sF3LToiG0wgez1FW5ctQPG9+OhL
+/tweKmKSdJMAnN2HoqRkRIgZmS998IaNnIfNCsJQ5jQttZQdG7gIeh027nR
4Vbovbgdsd2Gjp2HJNzFyblDjAncd3zkYvF2w9Tw1QTu7Zc0U9pD+u0X1GMW
q3nglY8Gjy9lYt3r7mS3d32Yeyvu2PCBfigpSndyumiwMx3/GHCbfO558WFP
9R50dRceMedTseB3kbbIExo2dqxzio1iQdHfdIUMmXtEiyW77GsoOPliETOY
0YxKdr7jgGMvcqrKOeIxdJLP7fcvrOtAuULvp2saH/C5Wztt3KMLA80ffXIZ
vWDJpvTrVlMRO8vVqnkdmXcylLWaSF25oMX8rrOBAcXQqkf51wQYpMyf6mDA
A9v3XMW7FiZiKm453XsgRLHe6ynCW1wYSZpVqmUSWPouUuuQvRBnZg7VqzK5
qBfv3jnVjdSTb0MP1NMF4BWnr14e2Yt+sUrvoQ9MqHf5rfgeyccq3aTFmys6
ke7yxaE/lgE+j2JxI4SHuPtuU5zKPkBySmO7USkN2z7FHtp5QghLtdkVWmVc
tMlqTTIh9TshO+CxA5kHd6nvPyKtwcOGH3k1efOY4F894SD06sdJ88YctTUD
uHPJYqFIJKkbdi9PWU5nY+rdpwrpm/qAww5Bh6VoOOt50NHTrBllVzIL9sqy
cU3Sv/91NQ0ZaYVPlcj6e2tULZwxoxuPaIMhjsvpOJ2VNnncsw2TaVejyop6
4Ce1rbNKlgq7z9veFuX14ZTNcuPj7v3Yu6C3hdVCQ9NCs1DHUh5e7Xwu59vN
RfJ+g+wf3QxYHEtvGeAJMFHQELxWnIcTyotGpaUJtJVHuxj9EcKta/scRzku
oko0vji3EdjxuiFaYjfJ89JYj+sz+pF9wk11eA8B9bigpC2bBHA5WVZvmdiD
lKu04zVZTExvtIncR/rA/Q0+Q+Mr2qF98NABedIHqovsudelwoOVl6G0WxwB
cYmwb0xVGoKDZ5+vaRHiWYZqht9ZLrht5hZVLwmoMK/0lJP9InlYfnahBMmH
oRI3LbJfvtzeKMldyIcEO3trkiwPrdPeLK2ZxUTZMM2QHcFBnpxny5eGfmzV
frc/bwEdXQbTTwyR/I52vGGt+KkPNqtcAzLsabivWSQv96cHQ6MXR+YGcFB+
+uayznAavMNkZu5T7YNsquEzD8l+VHkH7zhWRsPqS2dnrCocwECr3722rgF8
3rjxS/80BvYppLZulhcgr776/JNwHiovK24gwpi4dHJwxE9FiFlz6kqlSX94
et9onp4xgSwP06HpV4QQS/3kU1fWD4eJrwdbEgnE3x1bvIb0IdokX46rczBv
6pmNn/UJTLPTHFIQ8BEf0/+Fv4GNYpayrK4FE/eU1A7nMsh8+qmKWanPQnFT
TfCd7XRUbhteOZXMp6UfWbXOL5nIdteh2pP5NMXdZoVatRC7WdB4ZDcAxZKD
MzIKCRyZbHX5YYEQNu8DjGk3udjCMW69fpPAmbTBwD80AWxW23eakL7Oyf2H
9Ls/5Fz4M4uuvZKPFRL8yenKpM6wH1p5yJK58m08Q+ksD11mE2ckGrh4JOH7
fcl7Br6PSW+IXcODV6HEysc5XKwQ7xt6VMjA/ZLRzT+l+JCkBN0pnsXDVi/C
RHUqE6nzop/qupL89LRpPEz6Rhup4wlluUzMPyh6zeihAMdDj9qqLeXB++qt
8bsDTHj5J9dbiHzEN+/uxd6Hufih+mXS1HYCC3vexMvLC6Elu0QmXIkDi8SV
sr4rSR/42Se79rsA41Yx2ZSKPnwp+35/vTKBJefNApQD+TgQv3hOmmEn4gNL
LXR9GKALKqLS7pLv+XLTBqPJrXAFm9omTseWhmcznbtJX9u60VktngUXVZ1s
ykE67n7bvULtXzklPDN7iORbSGpqpguZUwIKCyKXvST7KCYvt9p0AEYTPtLV
D/6lV9fijZqECPJwds9P5EKqnfli7xMCV0+8yepxInNERkioHJULk6aRq/67
CIRMj/L5Q+b6vpNJXRfJftop41kuQeb6kIKCLF+OAOrzYzuKZvKQaOKUSJUg
cPqtl5GVhBDe7MLpYSNkDr65aMR9GQEN79xCRZJvbnLjB1I/c2F4482oHsk3
FoNiXf9TiNCcBb/dQ8g6nvnEmN1CIHmBhXFHgxAFC8ZY3y5yIfJgmPW4hIDd
nOj8hisCvL0eN7P2ODlPKdvVxpqZ8FQYSXv8mY8ZhazWWjc2fkZzw3/YMXHk
g2SWiTjpn3WWuDrmtCHHLIwm8ZyOlWrdl6u7eIhf67nAIpYFSheb3nSAjqzy
k+LcE1woyRuaJCyhYbR/UcL2P++x41GByJx4Uh/SBO7W5BxSdjkQS2liItm3
tX8JmX81F7AWTSHz71kt7UO2ZP4NvZCQUJctxCuZRdo3uf1oqJsa+jKdwBDT
qWnT3I9Y07JetN2XC6vyFpsiIYE3DbaNtOtCrPSWDIm7z8VAl7VLcTKBtabK
kyMlPiLmc+cuvQAuatW8g9M5BAb1zLNLu4TgMEZC9fMHcD/vZz6/hoBztHMJ
I06IO7x0vu8F0vfqRJ++Hk1Af87KKdazhRBZrh0bI86Bk/HOSAMypxhXXVH/
mipAYMf53CnHenFr9ZEud/I84zaeefnVVog3d05Zl9VyELK5WOKVKwFtexl1
NpnHr8g9dRcj87i2//U788k8/iUscZrxBj7iVJbJKpt1IOf5pW9FagycT/9Q
U15P8kkh+8RLAwaeVGFShNT/P4/PEVGUiD9A8ibBN35KJhUN36KCb9Dew4yv
jvdNfJw853s7pr4bN3xm3ByTYELndO+VhuUCHKZMXnp7aQ8WrnQZe32eibU/
wh33ZQlw7up0tdJbvchO2FDp0sUkc3DO8ZhJQnhoBWm4sfpQQcmPSFEnEPar
+sGwnhCT3SdEtUI4WNkb/bNyPQGhb0hskK4Qi9bwbVcf4uD9onUeuVYEiqRV
VmpKCvGgtC8sRZSDzeXZogXaBCTDJ+jCXtLf0vTkTx3tww8x+dGzZF98zRqs
657yETraXnbTlnHxIbhDRrWb/N2GN4u1hvlI8PPoovmz0Z3Af1m/lQmKsTvr
hDEfzkOBg7a8drzfqvvOfxYDhtd9v5pW8XHBImj6igPdKBodb183ykBQ8uKl
y8k80jRyJ/fSASYU1VJ0+x5RoXbtSN6Wx1wsiHtZUZdJxx2TkxeUqv/9+6jy
6VJJIXp85B6U6YzKb8eyMMuGrF90dBcnXWu6ycfKc7S0opQueLs97k94yYCn
wXDqnQo+FCsyXxS5d8N/7NGtZ18YyBh0DfabLEBD0Nph61tsFL59Zm5C7vee
4y+TaCMB/Bc56HjY9kDx4six2hQmVpSIVG/dI4CamM7arq4erKgMaqotYUKD
/jSRDyEeF88u/HGHA/tFc81jHQkcunHO9bWkAA/PGU0oVbMhN7x/zmPS38h1
Lhe3PcZHo2Sk6sVznTiqErAiNIyBgI2jCdul+Xh28fTWcVYbGvZTk8zq6OD6
qCvt0uBDz/lEOuHZjt5p1wPBp6PJNrWMPc7Fw5AhOYNEJuzu7J7kWEPOux5R
xc3WPMzJW5pmGt8MzbflhdP8/vP356Vcr8NDpH9lnG8sjJFohnbrn+UCKxpO
3vW4E0vOi8u3Q7L3JbAQ41qVrn6IjvI6a+Ptc/lQXWXm4C3ajk8bwlzLaXSo
bh3QNJHhw7XOiCnoboNR8tVf9W/p2Jga2HbZnI8F4ZG5cxU6sLKS4UdXYECb
EZI7kc9HYPLA0iRhF/aVquVkEAz02N7ZwgjgY1KW/cYMg06U3P4+lu/NwL0G
r1EVIQ+OcvHUK60spL3vMj8YTYdpsfoU+SIezDvk3+lZtSKSN1R5dB4dN1st
GQWjZF6eptHmptkG9C7ozUynI/DJYHliCxdpH+c5XrzKQKDa6ppNllTk3n/9
g0LmQKL7BcfgOhMpGYLFsxv+9+7D/118Vd7r9N9VXMRlaGm1/6BjKrVIWzBE
QYNFroJlK7nPy3UUp0wGIqso/W/XU9EU0sX4tpgPR/vPDqZH2tFzqHb9m0E6
orY8Spu6kQe708q/1mc349j2k+pnA0nfviw5vWInDx3i+ZOSFD9A1G9tw8kE
GkxzEte9muCh4EKUTeXONqSJ2mypuEtHm4f6Td/d5PpjFpLByz7Afa1IrloS
+Rzdi7MGFXnItip9kUrqu0vtuEmxHA03nBrfHSb91cDmcWd94xbEvHvyeXEz
DVcjVPQ/pZO+jTLly8yPNJyO2MRq8aXg/vFu1vO9XJi/rtJ45kBFQkAPz7ri
v/e/3l/8L/5PwKtD9I3EA0k/1xbUsSOcijibZeu63v7z9vmfxldnBeRcI33d
kvJ13zmkr4vw5h5ppv3z9vkX/4v/xf/i/2mcqmepMns3F6l1CzzmtFGQtmGf
5KSb/7x9/sX/Z3iazFsXzi4uFBa0JtS2UMDwkRnem/nP2+df/C/+F/+L/8X/
b+B/585f/C/+z8f/H9/i+40=
       "]], {
      PointSize[Large], 
      PointBox[
       NCache[{0, 
         Rational[-1, 3]}, {0, -0.3333333333333333}]]}, {{{{}, {}, 
        TagBox[{
          Directive[
           Opacity[1.], 
           RGBColor[0.368417, 0.506779, 0.709798], 
           AbsoluteThickness[1.6]], 
          LineBox[CompressedData["
1:eJw1mnk4Vd/3x0kyfIREiEpCSGXKFJYkJISQKSRNkiRExpBkJoTMZA6ZXZd9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           "]]}, Annotation[#, "Charting`Private`Tag$6102#1"]& ]}}, {}}}, {
    DisplayFunction -> Identity, AspectRatio -> 1, 
     AxesOrigin -> {Automatic, Automatic}, DisplayFunction :> Identity, Frame -> 
     True, FrameLabel -> {None, None}, 
     FrameTicks -> {{Automatic, Automatic}, {Automatic, Automatic}}, 
     GridLines -> {None, None}, GridLinesStyle -> Directive[
       GrayLevel[0.5, 0.4]], 
     Method -> {
      "DefaultBoundaryStyle" -> Automatic, "DefaultColorFunction" -> 
       "M10DefaultDensityGradient", 
       "DefaultGraphicsInteraction" -> {
        "Version" -> 1.2, "TrackMousePosition" -> {True, False}, 
         "Effects" -> {
          "Highlight" -> {"ratio" -> 2}, "HighlightPoint" -> {"ratio" -> 2}, 
           "Droplines" -> {
            "freeformCursorMode" -> True, 
             "placement" -> {"x" -> "All", "y" -> "None"}}}}, 
       "GridLinesInFront" -> True}, 
     PlotRange -> {{-0.3666666666666667, 0.3666666666666667}, {-0.55, 
      0.18333333333333335`}}, PlotRangeClipping -> True, PlotRangePadding -> {{
        Scaled[0.02], 
        Scaled[0.02]}, {
        Scaled[0.02], 
        Scaled[0.02]}}, Ticks -> {Automatic, Automatic}}], 
   FormBox[
    FormBox[
     TemplateBox[{
       FormBox[
        StyleBox[
         StyleBox[
          PaneBox[
           GraphicsBox[{
             RasterBox[CompressedData["
1:eJztl/k71F/YxxP6atGiLMmWSiRRSVK8UUm+qGQLbZZEQpFCaJESIls7ylJK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              "], {{
                Rational[-15, 2], 
                Rational[-225, 2]}, {
                Rational[15, 2], 
                Rational[225, 2]}}], {Antialiasing -> False, 
              AbsoluteThickness[0.1], 
              Directive[
               Opacity[0.3], 
               GrayLevel[0]], 
              LineBox[
               NCache[{{
                  Rational[15, 2], 
                  Rational[-225, 2]}, {
                  Rational[-15, 2], 
                  Rational[-225, 2]}, {
                  Rational[-15, 2], 
                  Rational[225, 2]}, {
                  Rational[15, 2], 
                  Rational[225, 2]}, {
                  Rational[15, 2], 
                  Rational[-225, 2]}}, {{7.5, -112.5}, {-7.5, -112.5}, {-7.5, 
                112.5}, {7.5, 112.5}, {7.5, -112.5}}]]}, {
              CapForm[None], {}}, 
             StyleBox[{Antialiasing -> False, 
               StyleBox[
                
                LineBox[{{7.500000000000001, -112.50000000000001`}, {
                 7.500000000000001, 112.50000000000001`}}], {
                 Directive[
                  AbsoluteThickness[0.2], 
                  Opacity[0.3], 
                  GrayLevel[0]]}, StripOnInput -> False], 
               StyleBox[
                StyleBox[{{
                   StyleBox[
                    LineBox[{{{7.500000000000001, -83.373786407767}, 
                    
                    Offset[{4., 0}, {
                    7.500000000000001, -83.373786407767}]}, {{
                    7.500000000000001, -46.966019417475735`}, 
                    
                    Offset[{4., 0}, {
                    7.500000000000001, -46.966019417475735`}]}, {{
                    7.500000000000001, -10.558252427184467`}, 
                    
                    Offset[{4., 0}, {
                    7.500000000000001, -10.558252427184467`}]}, {{
                    7.500000000000001, 25.849514563106798`}, 
                    
                    Offset[{4., 0}, {7.500000000000001, 
                    25.849514563106798`}]}, {{7.500000000000001, 
                    62.257281553398066`}, 
                    
                    Offset[{4., 0}, {7.500000000000001, 
                    62.257281553398066`}]}, {{7.500000000000001, 
                    98.66504854368934}, 
                    
                    Offset[{4., 0}, {7.500000000000001, 
                    98.66504854368934}]}}], {
                    Directive[
                    AbsoluteThickness[0.2], 
                    GrayLevel[0.4]]}, StripOnInput -> False], 
                   StyleBox[
                    LineBox[{{{7.500000000000001, -112.50000000000001`}, 
                    
                    Offset[{2.5, 0.}, {
                    7.500000000000001, -112.50000000000001`}]}, {{
                    7.500000000000001, -105.21844660194176`}, 
                    
                    Offset[{2.5, 0.}, {
                    7.500000000000001, -105.21844660194176`}]}, {{
                    7.500000000000001, -97.93689320388351}, 
                    
                    Offset[{2.5, 0.}, {
                    7.500000000000001, -97.93689320388351}]}, {{
                    7.500000000000001, -90.65533980582525}, 
                    
                    Offset[{2.5, 0.}, {
                    7.500000000000001, -90.65533980582525}]}, {{
                    7.500000000000001, -76.09223300970875}, 
                    
                    Offset[{2.5, 0.}, {
                    7.500000000000001, -76.09223300970875}]}, {{
                    7.500000000000001, -68.8106796116505}, 
                    
                    Offset[{2.5, 0.}, {
                    7.500000000000001, -68.8106796116505}]}, {{
                    7.500000000000001, -61.52912621359224}, 
                    
                    Offset[{2.5, 0.}, {
                    7.500000000000001, -61.52912621359224}]}, {{
                    7.500000000000001, -54.247572815533985`}, 
                    
                    Offset[{2.5, 0.}, {
                    7.500000000000001, -54.247572815533985`}]}, {{
                    7.500000000000001, -39.68446601941748}, 
                    
                    Offset[{2.5, 0.}, {
                    7.500000000000001, -39.68446601941748}]}, {{
                    7.500000000000001, -32.40291262135923}, 
                    
                    Offset[{2.5, 0.}, {
                    7.500000000000001, -32.40291262135923}]}, {{
                    7.500000000000001, -25.121359223300974`}, 
                    
                    Offset[{2.5, 0.}, {
                    7.500000000000001, -25.121359223300974`}]}, {{
                    7.500000000000001, -17.83980582524272}, 
                    
                    Offset[{2.5, 0.}, {
                    7.500000000000001, -17.83980582524272}]}, {{
                    7.500000000000001, -3.276699029126214}, 
                    
                    Offset[{2.5, 0.}, {
                    7.500000000000001, -3.276699029126214}]}, {{
                    7.500000000000001, 4.0048543689320395`}, 
                    
                    Offset[{2.5, 0.}, {7.500000000000001, 
                    4.0048543689320395`}]}, {{7.500000000000001, 
                    11.286407766990292`}, 
                    
                    Offset[{2.5, 0.}, {7.500000000000001, 
                    11.286407766990292`}]}, {{7.500000000000001, 
                    18.567961165048548`}, 
                    
                    Offset[{2.5, 0.}, {7.500000000000001, 
                    18.567961165048548`}]}, {{7.500000000000001, 
                    33.13106796116505}, 
                    
                    Offset[{2.5, 0.}, {7.500000000000001, 
                    33.13106796116505}]}, {{7.500000000000001, 
                    40.41262135922331}, 
                    
                    Offset[{2.5, 0.}, {7.500000000000001, 
                    40.41262135922331}]}, {{7.500000000000001, 
                    47.69417475728156}, 
                    
                    Offset[{2.5, 0.}, {7.500000000000001, 
                    47.69417475728156}]}, {{7.500000000000001, 
                    54.975728155339816`}, 
                    
                    Offset[{2.5, 0.}, {7.500000000000001, 
                    54.975728155339816`}]}, {{7.500000000000001, 
                    69.53883495145632}, 
                    Offset[{2.5, 0.}, {7.500000000000001, 
                    69.53883495145632}]}, {{7.500000000000001, 
                    76.82038834951457}, 
                    
                    Offset[{2.5, 0.}, {7.500000000000001, 
                    76.82038834951457}]}, {{7.500000000000001, 
                    84.10194174757282}, 
                    
                    Offset[{2.5, 0.}, {7.500000000000001, 
                    84.10194174757282}]}, {{7.500000000000001, 
                    91.38349514563107}, 
                    
                    Offset[{2.5, 0.}, {7.500000000000001, 
                    91.38349514563107}]}, {{7.500000000000001, 
                    105.94660194174759`}, 
                    
                    Offset[{2.5, 0.}, {7.500000000000001, 
                    105.94660194174759`}]}}], {
                    Directive[
                    AbsoluteThickness[0.2], 
                    GrayLevel[0.4], 
                    Opacity[0.3]]}, StripOnInput -> False]}, 
                  StyleBox[
                   StyleBox[{{
                    StyleBox[{
                    InsetBox[
                    FormBox[
                    RowBox[{"-", "30"}], TraditionalForm], 
                    Offset[{7., 0.}, {7.500000000000001, -83.373786407767}], 
                    ImageScaled[{0, 0.5}], Automatic, {1, 0}], 
                    InsetBox[
                    FormBox[
                    RowBox[{"-", "25"}], TraditionalForm], 
                    
                    Offset[{7., 0.}, {
                    7.500000000000001, -46.966019417475735`}], 
                    ImageScaled[{0, 0.5}], Automatic, {1, 0}], 
                    InsetBox[
                    FormBox[
                    RowBox[{"-", "20"}], TraditionalForm], 
                    
                    Offset[{7., 0.}, {
                    7.500000000000001, -10.558252427184467`}], 
                    ImageScaled[{0, 0.5}], Automatic, {1, 0}], 
                    InsetBox[
                    FormBox[
                    RowBox[{"-", "15"}], TraditionalForm], 
                    
                    Offset[{7., 0.}, {7.500000000000001, 
                    25.849514563106798`}], 
                    ImageScaled[{0, 0.5}], Automatic, {1, 0}], 
                    InsetBox[
                    FormBox[
                    RowBox[{"-", "10"}], TraditionalForm], 
                    
                    Offset[{7., 0.}, {7.500000000000001, 
                    62.257281553398066`}], 
                    ImageScaled[{0, 0.5}], Automatic, {1, 0}], 
                    InsetBox[
                    FormBox[
                    RowBox[{"-", "5"}], TraditionalForm], 
                    Offset[{7., 0.}, {7.500000000000001, 98.66504854368934}], 
                    ImageScaled[{0, 0.5}], Automatic, {1, 0}]}, {
                    Directive[
                    AbsoluteThickness[0.2], 
                    GrayLevel[0.4]], {
                    Directive[
                    Opacity[1]], 
                    Directive[
                    Opacity[1]]}}, StripOnInput -> False], 
                    
                    StyleBox[{{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, \
{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}}, {
                    Directive[
                    AbsoluteThickness[0.2], 
                    GrayLevel[0.4], 
                    Opacity[0.3]], {
                    Directive[
                    Opacity[1]], 
                    Directive[
                    Opacity[1]]}}, StripOnInput -> False]}, {}}, {{
                    Directive[
                    Opacity[1]], 
                    Directive[
                    Opacity[1]]}}, StripOnInput -> False], {"GraphicsLabel"}, 
                   StripOnInput -> False]}, {"GraphicsTicks"}, StripOnInput -> 
                 False], {{
                  Directive[
                   AbsoluteThickness[0.2], 
                   GrayLevel[0.4]], 
                  Directive[
                   AbsoluteThickness[0.2], 
                   GrayLevel[0.4], 
                   Opacity[0.3]]}}, StripOnInput -> False]}, {"GraphicsAxes"},
               StripOnInput -> False]}, PlotRangePadding -> Scaled[0.02], 
            PlotRange -> All, Frame -> True, 
            FrameTicks -> {{False, False}, {True, False}}, FrameStyle -> 
            Opacity[0], FrameTicksStyle -> Opacity[0], 
            ImageSize -> {Automatic, 225}, BaseStyle -> {}], Alignment -> 
           Left, AppearanceElements -> None, ImageMargins -> {{5, 5}, {5, 5}},
            ImageSizeAction -> "ResizeToFit"], LineIndent -> 0, StripOnInput -> 
          False], {FontFamily -> "Arial"}, Background -> Automatic, 
         StripOnInput -> False], TraditionalForm]}, "BarLegend", 
      DisplayFunction -> (#& ), 
      InterpretationFunction :> (RowBox[{"BarLegend", "[", 
         RowBox[{
           RowBox[{"{", 
             RowBox[{
               RowBox[{
                 RowBox[{
                   RowBox[{"(", 
                    RowBox[{
                    RowBox[{
                    RowBox[{"ColorData", "[", "\"DarkRainbow\"", "]"}], "[", 
                    RowBox[{"Rescale", "[", 
                    RowBox[{"#1", ",", 
                    RowBox[{"{", 
                    RowBox[{"minError", ",", "maxError"}], "}"}]}], "]"}], 
                    "]"}], "&"}], ")"}], "[", "#1", "]"}], "&"}], ",", 
               RowBox[{"{", 
                 RowBox[{
                   
                   RowBox[{
                    "-", "33.98651063280412581663552257873513683587`17.\
902238988470728"}], ",", 
                   
                   RowBox[{
                    "-", "3.\
0920056331802952144583942678939847344079447595005048658588`47.45151461143642"}\
]}], "}"}]}], "}"}], ",", 
           RowBox[{"LabelStyle", "\[Rule]", 
             RowBox[{"{", "}"}]}], ",", 
           RowBox[{"LegendLayout", "\[Rule]", "\"Column\""}], ",", 
           RowBox[{"LegendMarkerSize", "\[Rule]", "225"}], ",", 
           RowBox[{"ScalingFunctions", "\[Rule]", 
             RowBox[{"{", 
               RowBox[{"Identity", ",", "Identity"}], "}"}]}], ",", 
           RowBox[{"Charting`TickSide", "\[Rule]", "Right"}], ",", 
           RowBox[{"ColorFunctionScaling", "\[Rule]", "False"}]}], "]"}]& )], 
     TraditionalForm], TraditionalForm]},
  "Legended",
  DisplayFunction->(GridBox[{{
      TagBox[
       ItemBox[
        PaneBox[
         TagBox[#, "SkipImageSizeLevel"], Alignment -> {Center, Baseline}, 
         BaselinePosition -> Baseline], DefaultBaseStyle -> "Labeled"], 
       "SkipImageSizeLevel"], 
      ItemBox[#2, DefaultBaseStyle -> "LabeledLabel"]}}, 
    GridBoxAlignment -> {"Columns" -> {{Center}}, "Rows" -> {{Center}}}, 
    AutoDelete -> False, GridBoxItemSize -> Automatic, 
    BaselinePosition -> {1, 1}]& ),
  Editable->True,
  InterpretationFunction->(RowBox[{"Legended", "[", 
     RowBox[{#, ",", 
       RowBox[{"Placed", "[", 
         RowBox[{#2, ",", "After"}], "]"}]}], "]"}]& )]], "Output",
 CellChangeTimes->{{3.864478318195645*^9, 3.864478360310031*^9}, 
   3.864478687692378*^9, 3.864552383101451*^9, {3.864552451462137*^9, 
   3.8645524578885174`*^9}, 3.8645525410542965`*^9, 3.8645526365965147`*^9, 
   3.8645527273738947`*^9, 3.8645527942486963`*^9, 3.8645531417522535`*^9, 
   3.8645533382915287`*^9, 3.86609569664365*^9, 3.86609584584993*^9, 
   3.866096041082574*^9, 3.866096395871772*^9, 3.8660967332652693`*^9, 
   3.8660970471607695`*^9, 3.866097184300061*^9, {3.8660972773998413`*^9, 
   3.866097280970598*^9}, 3.86609731840666*^9, 3.8660977030523944`*^9, 
   3.8679205308900614`*^9, {3.867920651766673*^9, 3.8679206556158466`*^9}, {
   3.8679207889795876`*^9, 3.867920799149659*^9}, 3.86792568733837*^9},
 CellLabel->"Out[41]=",ExpressionUUID->"1c1f2f70-4d66-43bf-ad31-63aa68d723fb"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", 
   RowBox[{"**", 
    RowBox[{
    "*", " ", "Show", " ", "convergence", " ", "plot", " ", "of", " ", "QBX", 
     " ", "error", " ", "for", " ", "N", " ", "variation"}]}], " ", "****)"}],
   "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"x", "=", 
     RowBox[{"Subdivide", "[", 
      RowBox[{"0", ",", "n", ",", "n"}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{
     RowBox[{"maxN", "[", "nn_", "]"}], ":=", "\[IndentingNewLine]", 
     RowBox[{"Module", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"err", ",", "data"}], "}"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"err", "=", 
         RowBox[{"SLPERR", "[", "nn", "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"data", "=", 
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Re", "[", 
              RowBox[{"wevalPoints", "[", 
               RowBox[{"[", "idx", "]"}], "]"}], "]"}], ",", 
             RowBox[{"Im", "[", 
              RowBox[{"wevalPoints", "[", 
               RowBox[{"[", "idx", "]"}], "]"}], "]"}], ",", 
             RowBox[{"Abs", "[", 
              RowBox[{"err", "[", 
               RowBox[{"[", "idx", "]"}], "]"}], "]"}]}], "}"}], ",", 
           RowBox[{"{", 
            RowBox[{"idx", ",", "1", ",", 
             RowBox[{"Length", "[", "err", "]"}]}], "}"}]}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"max", "=", 
         RowBox[{"Max", "[", 
          RowBox[{
           RowBox[{"Select", "[", 
            RowBox[{"data", ",", 
             RowBox[{
              RowBox[{
               RowBox[{"#", "[", 
                RowBox[{"[", "2", "]"}], "]"}], "<", 
               RowBox[{"sc", "[", 
                RowBox[{"#", "[", 
                 RowBox[{"[", "1", "]"}], "]"}], "]"}]}], "&"}]}], "]"}], "[", 
           RowBox[{"[", 
            RowBox[{"All", ",", "3"}], "]"}], "]"}], "]"}]}]}]}], 
      "\[IndentingNewLine]", "]"}]}], ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"y", "=", 
     RowBox[{"ParallelTable", "[", 
      RowBox[{
       RowBox[{"maxN", "[", "nn", "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"nn", ",", "0", ",", "n"}], "}"}]}], "]"}]}], ";"}], 
   "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Export", "[", 
     RowBox[{"\"\<data/QBX/SLP/Nconvergence.mat\>\"", ",", "y"}], "]"}], 
    ";"}], "\[IndentingNewLine]", 
   RowBox[{"ListLogPlot", "[", 
    RowBox[{
     RowBox[{"Transpose", "@", 
      RowBox[{"{", 
       RowBox[{"x", ",", "y"}], "}"}]}], ",", 
     RowBox[{"Joined", "\[Rule]", "True"}]}], "]"}]}]}]], "Input",
 CellChangeTimes->{{3.862910496525565*^9, 3.862910560713912*^9}, {
   3.8629190232267685`*^9, 3.8629190373020544`*^9}, {3.8629192229405346`*^9, 
   3.862919223306241*^9}, 3.8629197015110397`*^9, {3.862921097076051*^9, 
   3.862921099014985*^9}, {3.8641129673519964`*^9, 3.864112969790104*^9}, {
   3.864318034084178*^9, 3.864318036542841*^9}, {3.866097717948118*^9, 
   3.8660977224479837`*^9}, {3.8679205521680746`*^9, 
   3.8679206111061687`*^9}, {3.867921242010722*^9, 3.8679212713288293`*^9}},
 CellLabel->"In[42]:=",ExpressionUUID->"c1c7e552-1b5c-48c9-a2e2-9305b8fd50fa"],

Cell[BoxData[
 GraphicsBox[{{}, {{}, {}, 
    {RGBColor[0.368417, 0.506779, 0.709798], PointSize[0.012833333333333334`],
      AbsoluteThickness[1.6], LineBox[CompressedData["
1:eJxTTMoPSmViYGAQBWIQDQPr9f8rSn3/vR/C+2B/6+tPmXc32A9ApR10zRI4
3yzjgPI5HCSTFvdkC/NA+QIOXA0/168+KAjlizisMuRfobFdFMqXcLBRuRBw
v1UYypdx6LJMyOaWE4HyFRx8/TRvms4Sg/KVHEpje4M+C0lC+SoO/O5VVq2H
xaF8NYf3B+8LTy2C8TUc3hiXrZ39VALK13IwjXs0P6ZIGsrXcVBnE/34pgnG
13NY1/f1eoeGFJRv4KDrHih0ZiWMb+iQ6W0pnJYgC+UbObCVpmfc95OH8o0d
NPgzmX/PkYHyTRwYjvGwVFXJHAAAEU9WSQ==
      "]]}}, {{}, {}}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->{True, True},
  AxesLabel->{None, None},
  AxesOrigin->{0, -8.30275573019822},
  DisplayFunction->Identity,
  Frame->{{False, False}, {False, False}},
  FrameLabel->{{None, None}, {None, None}},
  FrameTicks->{{
     Charting`ScaledTicks[{Log, Exp}], 
     Charting`ScaledFrameTicks[{Identity, Identity}]}, {Automatic, Automatic}},
  GridLines->{None, None},
  GridLinesStyle->Directive[
    GrayLevel[0.5, 0.4]],
  Method->{
   "OptimizePlotMarkers" -> True, "OptimizePlotMarkers" -> True, 
    "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
        Identity[
         Part[#, 1]], 
        Exp[
         Part[#, 2]]}& ), "CopiedValueFunction" -> ({
        Identity[
         Part[#, 1]], 
        Exp[
         Part[#, 2]]}& )}},
  PlotRange->{{0, 20.}, {-8.30275573019822, -1.7478276565552553`}},
  PlotRangeClipping->True,
  PlotRangePadding->{{
     Scaled[0.02], 
     Scaled[0.02]}, {
     Scaled[0.02], 
     Scaled[0.05]}},
  Ticks->FrontEndValueCache[{Automatic, 
     Charting`ScaledTicks[{Log, Exp}]}, {Automatic, {{-7.600902459542082, 
       FormBox[
        TemplateBox[{"\[Times]", "\"\[Times]\"", "5.`", 
          TemplateBox[{"10", 
            RowBox[{"-", "4"}]}, "Superscript", SyntaxForm -> 
           SuperscriptBox]}, "RowWithSeparators"], TraditionalForm], {0.01, 
       0.}}, {-6.907755278982137, 
       FormBox["0.001`", TraditionalForm], {0.01, 0.}}, {-5.298317366548036, 
       FormBox[
        TagBox[
         InterpretationBox[
          StyleBox["\"0.005\"", ShowStringCharacters -> False], 0.005, 
          AutoDelete -> True], NumberForm[#, {
           DirectedInfinity[1], 3}]& ], TraditionalForm], {0.01, 
       0.}}, {-4.605170185988091, 
       FormBox[
        TagBox[
         InterpretationBox[
          StyleBox["\"0.010\"", ShowStringCharacters -> False], 0.01, 
          AutoDelete -> True], NumberForm[#, {
           DirectedInfinity[1], 3}]& ], TraditionalForm], {0.01, 
       0.}}, {-2.995732273553991, 
       FormBox[
        TagBox[
         InterpretationBox[
          StyleBox["\"0.050\"", ShowStringCharacters -> False], 0.05, 
          AutoDelete -> True], NumberForm[#, {
           DirectedInfinity[1], 3}]& ], TraditionalForm], {0.01, 
       0.}}, {-2.3025850929940455`, 
       FormBox[
        TagBox[
         InterpretationBox[
          StyleBox["\"0.100\"", ShowStringCharacters -> False], 0.1, 
          AutoDelete -> True], NumberForm[#, {
           DirectedInfinity[1], 3}]& ], TraditionalForm], {0.01, 
       0.}}, {-9.210340371976182, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-8.517193191416238, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-8.111728083308073, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-7.824046010856292, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-7.418580902748128, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-7.264430222920869, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-7.1308988302963465`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-7.013115794639964, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-6.214608098422191, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-5.809142990314028, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-5.521460917862246, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-5.115995809754082, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-4.961845129926823, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-4.8283137373023015`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-4.710530701645918, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-3.912023005428146, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-3.506557897319982, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-3.2188758248682006`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-2.8134107167600364`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-2.659260036932778, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-2.5257286443082556`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-2.4079456086518722`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-1.6094379124341003`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-1.2039728043259361`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-0.916290731874155, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-0.6931471805599453, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-0.5108256237659907, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-0.35667494393873245`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-0.2231435513142097, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}, {-0.10536051565782628`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}}, {0., 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}}, {
      0.6931471805599453, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}}, {
      1.0986122886681098`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}}, {
      1.3862943611198906`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 0.}}, {
      1.6094379124341003`, 
       FormBox[
        TemplateBox[{0., 0.}, "Spacer2"], TraditionalForm], {0.005, 
       0.}}}}]]], "Output",
 CellChangeTimes->{
  3.862919704313911*^9, 3.8629200827483573`*^9, 3.8629211400930595`*^9, 
   3.862921408271866*^9, 3.862921814404378*^9, 3.862922292228512*^9, 
   3.863463090631181*^9, 3.864112995806452*^9, 3.8643180177701635`*^9, 
   3.864318058578053*^9, 3.864478370656201*^9, {3.864478458871126*^9, 
   3.8644784887933273`*^9}, {3.864478693954832*^9, 3.8644787049557924`*^9}, 
   3.8645524896510944`*^9, 3.8645526460950413`*^9, 3.864553343961211*^9, 
   3.866095717285291*^9, 3.866095851502528*^9, 3.866096046518077*^9, 
   3.8660964174051466`*^9, 3.8660967543424864`*^9, 3.8660973520390863`*^9, 
   3.8660977469541674`*^9, 3.8679206160865564`*^9, 3.867920801737874*^9, 
   3.867925689102702*^9},
 CellLabel->"Out[46]=",ExpressionUUID->"04a4b5d6-2a83-4673-8e22-cb21c5193177"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"(*", 
   RowBox[{"**", 
    RowBox[{"*", " ", "Export", " ", "remaining", " ", "data"}]}], " ", 
   "****)"}], "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"Export", "[", 
     RowBox[{"\"\<data/QBX/SLP/dataxy.mat\>\"", ",", "wevalPoints"}], "]"}], 
    ";"}], "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Export", "[", 
     RowBox[{"\"\<data/QBX/SLP/error.mat\>\"", ",", 
      RowBox[{"ParallelTable", "[", 
       RowBox[{
        RowBox[{
         RowBox[{"Log", "[", 
          RowBox[{"Abs", "[", 
           RowBox[{"SLPERR", "[", "nn", "]"}], "]"}], "]"}], "/", 
         RowBox[{"Log", "[", "10", "]"}]}], ",", 
        RowBox[{"{", 
         RowBox[{"nn", ",", "0", ",", "n"}], "}"}]}], "]"}]}], "]"}], 
    ";"}]}]}]], "Input",
 CellChangeTimes->{{3.862906801656907*^9, 3.862906840886513*^9}, {
   3.8629068779005575`*^9, 3.8629069077735486`*^9}, {3.862906953843136*^9, 
   3.8629069579089127`*^9}, {3.8629078696937017`*^9, 3.8629078854341803`*^9}, 
   3.86290796281295*^9, {3.8629079954333334`*^9, 3.8629080010503225`*^9}, {
   3.862910584410796*^9, 3.8629106385688953`*^9}, {3.8641130129841557`*^9, 
   3.86411301497078*^9}, {3.864318072812992*^9, 3.8643180745320516`*^9}, {
   3.86609775847626*^9, 3.8660977614955645`*^9}, {3.8679208143547397`*^9, 
   3.867920851379735*^9}, {3.86792127563968*^9, 3.867921279850258*^9}},
 CellLabel->"In[47]:=",ExpressionUUID->"1475a795-17cc-43aa-90e9-2fba69c76822"],

Cell[BoxData[""], "Input",
 CellChangeTimes->{{3.8629192696990843`*^9, 3.862919275011865*^9}},
 CellLabel->"In[49]:=",ExpressionUUID->"c04e5624-3fc1-4cb6-aa99-3ebb8d18582c"]
},
WindowSize->{1904, 991},
WindowMargins->{{-1920, Automatic}, {Automatic, 0}},
TaggingRules->{
 "WelcomeScreenSettings" -> {"FEStarting" -> False}, "TryRealOnly" -> False},
Magnification:>1.2 Inherited,
FrontEndVersion->"12.2 for Microsoft Windows (64-bit) (December 12, 2020)",
StyleDefinitions->"Default.nb",
ExpressionUUID->"ea102d2e-9dee-4b05-8256-86920c6c7617"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[558, 20, 6572, 134, 486, "Input",ExpressionUUID->"f82bcb07-fb1f-46be-9b92-f33ee38db7aa"],
Cell[7133, 156, 2741, 82, 180, "Input",ExpressionUUID->"69b602e8-6ddf-420a-96b8-feceb41bf3cf"],
Cell[CellGroupData[{
Cell[9899, 242, 5797, 142, 300, "Input",ExpressionUUID->"c120534d-ad3d-4b28-a52a-23a177e3226c"],
Cell[15699, 386, 1360, 19, 39, "Output",ExpressionUUID->"25ad1e88-a469-43d8-82d5-d1ceee50e4c0"],
Cell[17062, 407, 17303, 298, 288, "Output",ExpressionUUID->"59cb0c73-c673-4705-a669-bab6b32a264c"]
}, Open  ]],
Cell[34380, 708, 3316, 79, 198, "Input",ExpressionUUID->"a069f3ac-999e-4c5f-a3bd-770451c52dd9"],
Cell[37699, 789, 3711, 75, 149, "Input",ExpressionUUID->"212898ec-88d3-4f23-97a4-a339869bc2ba"],
Cell[41413, 866, 3737, 87, 156, "Input",ExpressionUUID->"c5efc32b-2405-4205-9699-e1dd2563ce85"],
Cell[45153, 955, 1550, 32, 84, "Input",ExpressionUUID->"f565b706-637e-4ec3-a5df-dd05b33bab01"],
Cell[CellGroupData[{
Cell[46728, 991, 7933, 180, 396, "Input",ExpressionUUID->"587aaa57-1a31-458b-9a74-f96f79916698"],
Cell[54664, 1173, 937, 14, 39, "Output",ExpressionUUID->"e727b0a1-d7be-47f9-97ca-3912a7e7b333"],
Cell[55604, 1189, 51924, 952, 444, "Output",ExpressionUUID->"1c1f2f70-4d66-43bf-ad31-63aa68d723fb"]
}, Open  ]],
Cell[CellGroupData[{
Cell[107565, 2146, 3272, 82, 276, "Input",ExpressionUUID->"c1c7e552-1b5c-48c9-a2e2-9305b8fd50fa"],
Cell[110840, 2230, 8117, 194, 274, "Output",ExpressionUUID->"04a4b5d6-2a83-4673-8e22-cb21c5193177"]
}, Open  ]],
Cell[118972, 2427, 1467, 32, 84, "Input",ExpressionUUID->"1475a795-17cc-43aa-90e9-2fba69c76822"],
Cell[120442, 2461, 173, 2, 35, "Input",ExpressionUUID->"c04e5624-3fc1-4cb6-aa99-3ebb8d18582c"]
}
]
*)

